@@ -1,8 +1,8 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
  * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
@@ -10,7 +10,7 @@
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sendf.c,v 1.81 2004/04/21 08:49:14 bagder Exp $
+ * $Id: sendf.c,v 1.93 2004/12/06 23:04:31 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -33,7 +33,7 @@
 #endif
 
 #ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>	/* required for send() & recv() prototypes */
+#include <sys/socket.h> /* required for send() & recv() prototypes */
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -52,15 +52,15 @@
 #include "security.h"
 #endif
 #include <string.h>
+#include "memory.h"
+#include "strerror.h"
 /* The last #include file should be: */
-#ifdef CURLDEBUG
 #include "memdebug.h"
-#endif
 
 /* returns last node in linked list */
 static struct curl_slist *slist_get_last(struct curl_slist *list)
 {
-  struct curl_slist	*item;
+  struct curl_slist     *item;
 
   /* if caller passed us a NULL, return now */
   if (!list)
@@ -74,25 +74,33 @@ static struct curl_slist *slist_get_last(struct curl_slist *list)
   return item;
 }
 
-/* append a struct to the linked list. It always retunrs the address of the
- * first record, so that you can sure this function as an initialization
- * function as well as an append function. If you find this bothersome,
- * then simply create a separate _init function and call it appropriately from
- * within the proram. */
+/*
+ * curl_slist_append() appends a string to the linked list. It always retunrs
+ * the address of the first record, so that you can sure this function as an
+ * initialization function as well as an append function. If you find this
+ * bothersome, then simply create a separate _init function and call it
+ * appropriately from within the proram.
+ */
 struct curl_slist *curl_slist_append(struct curl_slist *list,
                                      const char *data)
 {
-  struct curl_slist	*last;
-  struct curl_slist	*new_item;
+  struct curl_slist     *last;
+  struct curl_slist     *new_item;
 
   new_item = (struct curl_slist *) malloc(sizeof(struct curl_slist));
   if (new_item) {
-    new_item->next = NULL;
-    new_item->data = strdup(data);
+    char *dup = strdup(data);
+    if(dup) {
+      new_item->next = NULL;
+      new_item->data = dup;
+    }
+    else {
+      free(new_item);
+      return NULL;
+    }
   }
-  if (new_item == NULL || new_item->data == NULL) {
+  else
     return NULL;
-  }
 
   if (list) {
     last = slist_get_last(list);
@@ -107,8 +115,8 @@ struct curl_slist *curl_slist_append(struct curl_slist *list,
 /* be nice and clean up resources */
 void curl_slist_free_all(struct curl_slist *list)
 {
-  struct curl_slist	*next;
-  struct curl_slist	*item;
+  struct curl_slist     *next;
+  struct curl_slist     *item;
 
   if (!list)
     return;
@@ -116,7 +124,7 @@ void curl_slist_free_all(struct curl_slist *list)
   item = list;
   do {
     next = item->next;
-		
+
     if (item->data) {
       free(item->data);
     }
@@ -135,7 +143,7 @@ void Curl_infof(struct SessionHandle *data, const char *fmt, ...)
     va_start(ap, fmt);
     vsnprintf(print_buffer, 1024, fmt, ap);
     va_end(ap);
-    Curl_debug(data, CURLINFO_TEXT, print_buffer, strlen(print_buffer));
+    Curl_debug(data, CURLINFO_TEXT, print_buffer, strlen(print_buffer), NULL);
   }
 }
 
@@ -150,21 +158,20 @@ void Curl_failf(struct SessionHandle *data, const char *fmt, ...)
   if(data->set.errorbuffer && !data->state.errorbuf) {
     vsnprintf(data->set.errorbuffer, CURL_ERROR_SIZE, fmt, ap);
     data->state.errorbuf = TRUE; /* wrote error string */
-
-    if(data->set.verbose) {
-      size_t len = strlen(data->set.errorbuffer);
-      bool doneit=FALSE;
-      if(len < CURL_ERROR_SIZE - 1) {
-        doneit = TRUE;
-        data->set.errorbuffer[len] = '\n';
-        data->set.errorbuffer[++len] = '\0';
-      }
-      Curl_debug(data, CURLINFO_TEXT, data->set.errorbuffer, len);
-      if(doneit)
-        /* cut off the newline again */
-        data->set.errorbuffer[--len]=0;
-    }
   }
+  if(data->set.verbose) {
+      size_t len;
+
+      vsnprintf(data->state.buffer, BUFSIZE, fmt, ap);
+      len = strlen(data->state.buffer);
+
+      if(len < BUFSIZE - 1) {
+        data->state.buffer[len] = '\n';
+        data->state.buffer[++len] = '\0';
+      }
+      Curl_debug(data, CURLINFO_TEXT, data->state.buffer, len, NULL);
+  }
+
   va_end(ap);
 }
 
@@ -197,7 +204,8 @@ CURLcode Curl_sendf(curl_socket_t sockfd, struct connectdata *conn,
       break;
 
     if(data->set.verbose)
-      Curl_debug(data, CURLINFO_DATA_OUT, sptr, bytes_written);
+      Curl_debug(data, CURLINFO_DATA_OUT, sptr, bytes_written,
+                 conn->host.dispname);
 
     if((size_t)bytes_written != write_len) {
       /* if not all was written at once, we must advance the pointer, decrease
@@ -238,12 +246,12 @@ CURLcode Curl_write(struct connectdata *conn,
     int err;
     char error_buffer[120]; /* OpenSSL documents that this must be at least
                                120 bytes long. */
-    int sslerror;
-    int rc = SSL_write(conn->ssl[num].handle, mem, len);
+    unsigned long sslerror;
+    int rc = SSL_write(conn->ssl[num].handle, mem, (int)len);
 
     if(rc < 0) {
       err = SSL_get_error(conn->ssl[num].handle, rc);
-    
+
       switch(err) {
       case SSL_ERROR_WANT_READ:
       case SSL_ERROR_WANT_WRITE:
@@ -300,6 +308,9 @@ CURLcode Curl_write(struct connectdata *conn,
         )
         /* this is just a case of EWOULDBLOCK */
         bytes_written=0;
+      else
+        failf(conn->data, "Send failure: %s",
+              Curl_strerror(conn, err));
     }
 #ifdef USE_SSLEAY
   }
@@ -348,7 +359,7 @@ CURLcode Curl_client_write(struct SessionHandle *data,
       return CURLE_WRITE_ERROR;
     }
   }
-  
+
   return CURLE_OK;
 }
 
@@ -375,11 +386,11 @@ int Curl_read(struct connectdata *conn, /* connection data */
   *n=0; /* reset amount to zero */
 
   if (conn->ssl[num].use) {
-    nread = SSL_read(conn->ssl[num].handle, buf, buffersize);
+    nread = (ssize_t)SSL_read(conn->ssl[num].handle, buf, (int)buffersize);
 
     if(nread < 0) {
       /* failed SSL_read */
-      int err = SSL_get_error(conn->ssl[num].handle, nread);
+      int err = SSL_get_error(conn->ssl[num].handle, (int)nread);
 
       switch(err) {
       case SSL_ERROR_NONE: /* this is not an error */
@@ -394,7 +405,7 @@ int Curl_read(struct connectdata *conn, /* connection data */
         {
           char error_buffer[120]; /* OpenSSL documents that this must be at
                                      least 120 bytes long. */
-          int sslerror = ERR_get_error();
+          unsigned long sslerror = ERR_get_error();
           failf(conn->data, "SSL read: %s, errno %d",
                 ERR_error_string(sslerror, error_buffer),
                 Curl_ourerrno() );
@@ -433,11 +444,11 @@ int Curl_read(struct connectdata *conn, /* connection data */
 }
 
 /* return 0 on success */
-int Curl_debug(struct SessionHandle *data, curl_infotype type,
-               char *ptr, size_t size)
+static int showit(struct SessionHandle *data, curl_infotype type,
+                  char *ptr, size_t size)
 {
   static const char * const s_infotype[CURLINFO_END] = {
-    "* ", "< ", "> ", "{ ", "} " };
+    "* ", "< ", "> ", "{ ", "} ", "{ ", "} " };
 
   if(data->set.fdebug)
     return (*data->set.fdebug)(data, type, ptr, size,
@@ -455,3 +466,34 @@ int Curl_debug(struct SessionHandle *data, curl_infotype type,
   }
   return 0;
 }
+
+int Curl_debug(struct SessionHandle *data, curl_infotype type,
+               char *ptr, size_t size, char *host)
+{
+  int rc;
+  if(data->set.printhost && host) {
+    char buffer[160];
+    const char *t=NULL;
+    switch (type) {
+    case CURLINFO_HEADER_IN:
+    case CURLINFO_DATA_IN:
+      t = "from";
+      break;
+    case CURLINFO_HEADER_OUT:
+    case CURLINFO_DATA_OUT:
+      t = "to";
+      break;
+    default:
+      break;
+    }
+
+    if(t) {
+      snprintf(buffer, sizeof(buffer), "[Data %s %s]", t, host);
+      rc = showit(data, CURLINFO_TEXT, buffer, strlen(buffer));
+      if(rc)
+        return rc;
+    }
+  }
+  rc = showit(data, type, ptr, size);
+  return rc;
+}
