@@ -1,16 +1,16 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: telnet.c,v 1.40 2003/03/12 08:44:00 bagder Exp $
+ * $Id: telnet.c,v 1.71 2004/11/19 08:52:33 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -30,13 +30,15 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-
+#endif
 #include <errno.h>
 
-#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
-#include <winsock2.h>
+#if defined(WIN32)
 #include <time.h>
 #include <io.h>
 #else
@@ -45,7 +47,6 @@
 #endif
 #include <netinet/in.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -63,10 +64,6 @@
 #include <sys/param.h>
 #endif
 
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
 
 #endif
 
@@ -74,6 +71,7 @@
 #include <curl/curl.h>
 #include "transfer.h"
 #include "sendf.h"
+#include "telnet.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -82,11 +80,11 @@
 #define  TELCMDS
 
 #include "arpa_telnet.h"
+#include "memory.h"
+#include "select.h"
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
 #include "memdebug.h"
-#endif
 
 #define SUBBUFSIZE 512
 
@@ -102,14 +100,19 @@
 #define  CURL_SB_EOF(x) (x->subpointer >= x->subend)
 #define  CURL_SB_LEN(x) (x->subend - x->subpointer)
 
+#ifdef WIN32
+typedef FARPROC WSOCK2_FUNC;
+static CURLcode check_wsock2 ( struct SessionHandle *data );
+#endif
+
 static
 void telrcv(struct connectdata *,
-	    unsigned char *inbuf,	/* Data received from socket */
-	    int count);			/* Number of bytes received */
+            unsigned char *inbuf,       /* Data received from socket */
+            ssize_t count);             /* Number of bytes received */
 
 static void printoption(struct SessionHandle *data,
-			const char *direction,
-			int cmd, int option);
+                        const char *direction,
+                        int cmd, int option);
 
 static void negotiate(struct connectdata *);
 static void send_negotiation(struct connectdata *, int cmd, int option);
@@ -117,16 +120,17 @@ static void set_local_option(struct connectdata *, int cmd, int option);
 static void set_remote_option(struct connectdata *, int cmd, int option);
 
 static void printsub(struct SessionHandle *data,
-		     int direction, unsigned char *pointer, int length);
+                     int direction, unsigned char *pointer,
+                     size_t length);
 static void suboption(struct connectdata *);
 
 /* For negotiation compliant to RFC 1143 */
-#define CURL_NO	         0
-#define CURL_YES 	 1
-#define CURL_WANTYES	 2
-#define CURL_WANTNO	 3
+#define CURL_NO          0
+#define CURL_YES         1
+#define CURL_WANTYES     2
+#define CURL_WANTNO      3
 
-#define CURL_EMPTY	 0
+#define CURL_EMPTY       0
 #define CURL_OPPOSITE    1
 
 /*
@@ -148,12 +152,12 @@ typedef enum
 struct TELNET {
   int please_negotiate;
   int already_negotiated;
-  int us[256]; 
-  int usq[256]; 
-  int us_preferred[256]; 
-  int him[256]; 
-  int himq[256]; 
-  int him_preferred[256]; 
+  int us[256];
+  int usq[256];
+  int us_preferred[256];
+  int him[256];
+  int himq[256];
+  int him_preferred[256];
   char subopt_ttype[32];             /* Set with suboption TTYPE */
   char subopt_xdisploc[128];          /* Set with suboption XDISPLOC */
   struct curl_slist *telnet_vars; /* Environment variables */
@@ -161,38 +165,65 @@ struct TELNET {
   /* suboptions */
   char subbuffer[SUBBUFSIZE];
   char *subpointer, *subend;      /* buffer for sub-options */
-  
+
   TelnetReceive telrcv_state;
 };
 
+#ifdef WIN32
+static CURLcode
+check_wsock2 ( struct SessionHandle *data )
+{
+  int err;
+  WORD wVersionRequested;
+  WSADATA wsaData;
+
+  curlassert(data);
+
+  /* telnet requires at least WinSock 2.0 so ask for it. */
+  wVersionRequested = MAKEWORD(2, 0);
+
+  err = WSAStartup(wVersionRequested, &wsaData);
+
+  /* We must've called this once already, so this call */
+  /* should always succeed.  But, just in case... */
+  if (err != 0) {
+    failf(data,"WSAStartup failed (%d)",err);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* We have to have a WSACleanup call for every successful */
+  /* WSAStartup call. */
+  WSACleanup();
+
+  /* Check that our version is supported */
+  if (LOBYTE(wsaData.wVersion) != LOBYTE(wVersionRequested) ||
+      HIBYTE(wsaData.wVersion) != HIBYTE(wVersionRequested)) {
+      /* Our version isn't supported */
+      failf(data,"insufficient winsock version to support "
+            "telnet");
+      return CURLE_FAILED_INIT;
+  }
+
+  /* Our version is supported */
+  return CURLE_OK;
+}
+#endif
 static
 CURLcode init_telnet(struct connectdata *conn)
 {
   struct TELNET *tn;
 
-  tn = (struct TELNET *)malloc(sizeof(struct TELNET));
+  tn = (struct TELNET *)calloc(1, sizeof(struct TELNET));
   if(!tn)
     return CURLE_OUT_OF_MEMORY;
-  
-  conn->proto.telnet = (void *)tn; /* make us known */
 
-  memset(tn, 0, sizeof(struct TELNET));
+  conn->proto.telnet = (void *)tn; /* make us known */
 
   tn->telrcv_state = CURL_TS_DATA;
 
   /* Init suboptions */
   CURL_SB_CLEAR(tn);
 
-  /* Set all options to NO */
-#if 0
-  /* NO is zero => default fill pattern */
-  memset(tn->us, CURL_NO, 256);
-  memset(tn->usq, CURL_NO, 256);
-  memset(tn->us_preferred, CURL_NO, 256);
-  memset(tn->him, CURL_NO, 256);
-  memset(tn->himq, CURL_NO, 256);
-  memset(tn->him_preferred, CURL_NO, 256);
-#endif
   /* Set the options we want by default */
   tn->us_preferred[CURL_TELOPT_BINARY] = CURL_YES;
   tn->us_preferred[CURL_TELOPT_SGA] = CURL_YES;
@@ -206,31 +237,31 @@ static void negotiate(struct connectdata *conn)
 {
   int i;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
-   
+
   for(i = 0;i < CURL_NTELOPTS;i++)
   {
     if(tn->us_preferred[i] == CURL_YES)
       set_local_option(conn, i, CURL_YES);
-      
+
     if(tn->him_preferred[i] == CURL_YES)
       set_remote_option(conn, i, CURL_YES);
   }
 }
 
 static void printoption(struct SessionHandle *data,
-			const char *direction, int cmd, int option)
+                        const char *direction, int cmd, int option)
 {
   const char *fmt;
   const char *opt;
-   
+
   if (data->set.verbose)
   {
     if (cmd == CURL_IAC)
     {
       if (CURL_TELCMD_OK(option))
-        Curl_infof(data, "%s IAC %s\n", direction, CURL_TELCMD(option));
+        infof(data, "%s IAC %s\n", direction, CURL_TELCMD(option));
       else
-        Curl_infof(data, "%s IAC %d\n", direction, option);
+        infof(data, "%s IAC %d\n", direction, option);
     }
     else
     {
@@ -246,12 +277,12 @@ static void printoption(struct SessionHandle *data,
           opt = NULL;
 
         if(opt)
-          Curl_infof(data, "%s %s %s\n", direction, fmt, opt);
+          infof(data, "%s %s %s\n", direction, fmt, opt);
         else
-          Curl_infof(data, "%s %s %d\n", direction, fmt, option);
+          infof(data, "%s %s %d\n", direction, fmt, option);
       }
       else
-        Curl_infof(data, "%s %d %d\n", direction, cmd, option);
+        infof(data, "%s %d %d\n", direction, cmd, option);
     }
   }
 }
@@ -263,9 +294,9 @@ static void send_negotiation(struct connectdata *conn, int cmd, int option)
    buf[0] = CURL_IAC;
    buf[1] = cmd;
    buf[2] = option;
-   
-   (void)swrite(conn->firstsocket, buf, 3);
-   
+
+   (void)swrite(conn->sock[FIRSTSOCKET], (char *)buf, 3);
+
    printoption(conn->data, "SENT", cmd, option);
 }
 
@@ -281,11 +312,11 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
         tn->him[option] = CURL_WANTYES;
         send_negotiation(conn, CURL_DO, option);
         break;
-	 
+
       case CURL_YES:
         /* Already enabled */
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->himq[option])
         {
@@ -298,7 +329,7 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->himq[option])
         {
@@ -319,12 +350,12 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
       case CURL_NO:
         /* Already disabled */
         break;
-	 
+
       case CURL_YES:
         tn->him[option] = CURL_WANTNO;
         send_negotiation(conn, CURL_DONT, option);
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->himq[option])
         {
@@ -336,7 +367,7 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->himq[option])
         {
@@ -368,11 +399,11 @@ void rec_will(struct connectdata *conn, int option)
         send_negotiation(conn, CURL_DONT, option);
       }
       break;
-	 
+
     case CURL_YES:
       /* Already enabled */
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->himq[option])
       {
@@ -387,7 +418,7 @@ void rec_will(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->himq[option])
       {
@@ -403,7 +434,7 @@ void rec_will(struct connectdata *conn, int option)
       break;
   }
 }
-   
+
 static
 void rec_wont(struct connectdata *conn, int option)
 {
@@ -413,19 +444,19 @@ void rec_wont(struct connectdata *conn, int option)
     case CURL_NO:
       /* Already disabled */
       break;
-	 
+
     case CURL_YES:
       tn->him[option] = CURL_NO;
       send_negotiation(conn, CURL_DONT, option);
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->himq[option])
       {
         case CURL_EMPTY:
           tn->him[option] = CURL_NO;
           break;
-	 
+
         case CURL_OPPOSITE:
           tn->him[option] = CURL_WANTYES;
           tn->himq[option] = CURL_EMPTY;
@@ -433,7 +464,7 @@ void rec_wont(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->himq[option])
       {
@@ -448,7 +479,7 @@ void rec_wont(struct connectdata *conn, int option)
       break;
   }
 }
-   
+
 static void
 set_local_option(struct connectdata *conn, int option, int newstate)
 {
@@ -461,11 +492,11 @@ set_local_option(struct connectdata *conn, int option, int newstate)
         tn->us[option] = CURL_WANTYES;
         send_negotiation(conn, CURL_WILL, option);
         break;
-	 
+
       case CURL_YES:
         /* Already enabled */
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->usq[option])
         {
@@ -478,7 +509,7 @@ set_local_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->usq[option])
         {
@@ -499,12 +530,12 @@ set_local_option(struct connectdata *conn, int option, int newstate)
       case CURL_NO:
         /* Already disabled */
         break;
-	 
+
       case CURL_YES:
         tn->us[option] = CURL_WANTNO;
         send_negotiation(conn, CURL_WONT, option);
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->usq[option])
         {
@@ -516,7 +547,7 @@ set_local_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->usq[option])
         {
@@ -548,11 +579,11 @@ void rec_do(struct connectdata *conn, int option)
         send_negotiation(conn, CURL_WONT, option);
       }
       break;
-	 
+
     case CURL_YES:
       /* Already enabled */
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->usq[option])
       {
@@ -567,7 +598,7 @@ void rec_do(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->usq[option])
       {
@@ -584,7 +615,7 @@ void rec_do(struct connectdata *conn, int option)
   }
 }
 
-static   
+static
 void rec_dont(struct connectdata *conn, int option)
 {
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
@@ -593,19 +624,19 @@ void rec_dont(struct connectdata *conn, int option)
     case CURL_NO:
       /* Already disabled */
       break;
-	 
+
     case CURL_YES:
       tn->us[option] = CURL_NO;
       send_negotiation(conn, CURL_WONT, option);
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->usq[option])
       {
         case CURL_EMPTY:
           tn->us[option] = CURL_NO;
           break;
-	 
+
         case CURL_OPPOSITE:
           tn->us[option] = CURL_WANTYES;
           tn->usq[option] = CURL_EMPTY;
@@ -613,7 +644,7 @@ void rec_dont(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->usq[option])
       {
@@ -631,17 +662,17 @@ void rec_dont(struct connectdata *conn, int option)
 
 
 static void printsub(struct SessionHandle *data,
-		     int direction,		/* '<' or '>' */
-		     unsigned char *pointer,	/* where suboption data is */
-		     int length)		/* length of suboption data */
+                     int direction,             /* '<' or '>' */
+                     unsigned char *pointer,    /* where suboption data is */
+                     size_t length)             /* length of suboption data */
 {
-  int i = 0;
+  unsigned int i = 0;
 
   if (data->set.verbose)
   {
     if (direction)
     {
-      Curl_infof(data, "%s IAC SB ", (direction == '<')? "RCVD":"SENT");
+      infof(data, "%s IAC SB ", (direction == '<')? "RCVD":"SENT");
       if (length >= 3)
       {
         int j;
@@ -651,27 +682,27 @@ static void printsub(struct SessionHandle *data,
 
         if (i != CURL_IAC || j != CURL_SE)
         {
-          Curl_infof(data, "(terminated by ");
+          infof(data, "(terminated by ");
           if (CURL_TELOPT_OK(i))
-            Curl_infof(data, "%s ", CURL_TELOPT(i));
+            infof(data, "%s ", CURL_TELOPT(i));
           else if (CURL_TELCMD_OK(i))
-            Curl_infof(data, "%s ", CURL_TELCMD(i));
+            infof(data, "%s ", CURL_TELCMD(i));
           else
-            Curl_infof(data, "%d ", i);
+            infof(data, "%d ", i);
           if (CURL_TELOPT_OK(j))
-            Curl_infof(data, "%s", CURL_TELOPT(j));
+            infof(data, "%s", CURL_TELOPT(j));
           else if (CURL_TELCMD_OK(j))
-            Curl_infof(data, "%s", CURL_TELCMD(j));
+            infof(data, "%s", CURL_TELCMD(j));
           else
-            Curl_infof(data, "%d", j);
-          Curl_infof(data, ", not IAC SE!) ");
+            infof(data, "%d", j);
+          infof(data, ", not IAC SE!) ");
         }
       }
       length -= 2;
     }
     if (length < 1)
     {
-      Curl_infof(data, "(Empty suboption?)");
+      infof(data, "(Empty suboption?)");
       return;
     }
 
@@ -680,50 +711,50 @@ static void printsub(struct SessionHandle *data,
         case CURL_TELOPT_TTYPE:
         case CURL_TELOPT_XDISPLOC:
         case CURL_TELOPT_NEW_ENVIRON:
-          Curl_infof(data, "%s", CURL_TELOPT(pointer[0]));
+          infof(data, "%s", CURL_TELOPT(pointer[0]));
           break;
         default:
-          Curl_infof(data, "%s (unsupported)", CURL_TELOPT(pointer[0]));
+          infof(data, "%s (unsupported)", CURL_TELOPT(pointer[0]));
           break;
       }
     }
     else
-      Curl_infof(data, "%d (unknown)", pointer[i]);
+      infof(data, "%d (unknown)", pointer[i]);
 
     switch(pointer[1]) {
       case CURL_TELQUAL_IS:
-        Curl_infof(data, " IS");
+        infof(data, " IS");
         break;
       case CURL_TELQUAL_SEND:
-        Curl_infof(data, " SEND");
+        infof(data, " SEND");
         break;
       case CURL_TELQUAL_INFO:
-        Curl_infof(data, " INFO/REPLY");
+        infof(data, " INFO/REPLY");
         break;
       case CURL_TELQUAL_NAME:
-        Curl_infof(data, " NAME");
+        infof(data, " NAME");
         break;
     }
-      
+
     switch(pointer[0]) {
       case CURL_TELOPT_TTYPE:
       case CURL_TELOPT_XDISPLOC:
         pointer[length] = 0;
-        Curl_infof(data, " \"%s\"", &pointer[2]);
+        infof(data, " \"%s\"", &pointer[2]);
         break;
       case CURL_TELOPT_NEW_ENVIRON:
         if(pointer[1] == CURL_TELQUAL_IS) {
-          Curl_infof(data, " ");
+          infof(data, " ");
           for(i = 3;i < length;i++) {
             switch(pointer[i]) {
               case CURL_NEW_ENV_VAR:
-                Curl_infof(data, ", ");
+                infof(data, ", ");
                 break;
               case CURL_NEW_ENV_VALUE:
-                Curl_infof(data, " = ");
+                infof(data, " = ");
                 break;
               default:
-                Curl_infof(data, "%c", pointer[i]);
+                infof(data, "%c", pointer[i]);
                 break;
             }
           }
@@ -731,18 +762,18 @@ static void printsub(struct SessionHandle *data,
         break;
       default:
         for (i = 2; i < length; i++)
-          Curl_infof(data, " %.2x", pointer[i]);
+          infof(data, " %.2x", pointer[i]);
         break;
     }
-      
+
     if (direction)
     {
-      Curl_infof(data, "\n");
+      infof(data, "\n");
     }
   }
 }
 
-static int check_telnet_options(struct connectdata *conn)
+static CURLcode check_telnet_options(struct connectdata *conn)
 {
   struct curl_slist *head;
   char option_keyword[128];
@@ -755,9 +786,8 @@ static int check_telnet_options(struct connectdata *conn)
      was given on the command line */
   if(conn->bits.user_passwd)
   {
-    char *buf = malloc(256);
-    sprintf(buf, "USER,%s", data->state.user);
-    tn->telnet_vars = curl_slist_append(tn->telnet_vars, buf);
+    snprintf(option_arg, sizeof(option_arg), "USER,%s", conn->user);
+    tn->telnet_vars = curl_slist_append(tn->telnet_vars, option_arg);
 
     tn->us_preferred[CURL_TELOPT_NEW_ENVIRON] = CURL_YES;
   }
@@ -813,23 +843,22 @@ static int check_telnet_options(struct connectdata *conn)
 static void suboption(struct connectdata *conn)
 {
   struct curl_slist *v;
-  unsigned char subchar;
   unsigned char temp[2048];
-  int len;
-  int tmplen;
+  size_t len;
+  size_t tmplen;
   char varname[128];
   char varval[128];
   struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   printsub(data, '<', (unsigned char *)tn->subbuffer, CURL_SB_LEN(tn)+2);
-  switch (subchar = CURL_SB_GET(tn)) {
+  switch (CURL_SB_GET(tn)) {
     case CURL_TELOPT_TTYPE:
       len = strlen(tn->subopt_ttype) + 4 + 2;
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_TTYPE,
                CURL_TELQUAL_IS, tn->subopt_ttype, CURL_IAC, CURL_SE);
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], (char *)temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_XDISPLOC:
@@ -837,7 +866,7 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_XDISPLOC,
                CURL_TELQUAL_IS, tn->subopt_xdisploc, CURL_IAC, CURL_SE);
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], (char *)temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_NEW_ENVIRON:
@@ -850,7 +879,7 @@ static void suboption(struct connectdata *conn)
         tmplen = (strlen(v->data) + 1);
         /* Add the variable only if it fits */
         if(len + tmplen < (int)sizeof(temp)-6) {
-          sscanf(v->data, "%127[^,],%s", varname, varval);
+          sscanf(v->data, "%127[^,],%127s", varname, varval);
           snprintf((char *)&temp[len], sizeof(temp) - len,
                    "%c%s%c%s", CURL_NEW_ENV_VAR, varname,
                    CURL_NEW_ENV_VALUE, varval);
@@ -860,7 +889,7 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)&temp[len], sizeof(temp) - len,
                "%c%c", CURL_IAC, CURL_SE);
       len += 2;
-      (void)swrite(conn->firstsocket, temp, len);
+      (void)swrite(conn->sock[FIRSTSOCKET], (char *)temp, len);
       printsub(data, '>', &temp[2], len-2);
       break;
   }
@@ -869,17 +898,17 @@ static void suboption(struct connectdata *conn)
 
 static
 void telrcv(struct connectdata *conn,
-            unsigned char *inbuf,	/* Data received from socket */
-            int count)			/* Number of bytes received */
+            unsigned char *inbuf,       /* Data received from socket */
+            ssize_t count)              /* Number of bytes received */
 {
   unsigned char c;
-  int index = 0;
+  int in = 0;
   struct SessionHandle *data = conn->data;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
 
   while(count--)
   {
-    c = inbuf[index++];
+    c = inbuf[in++];
 
     switch (tn->telrcv_state)
     {
@@ -946,21 +975,21 @@ void telrcv(struct connectdata *conn,
         rec_will(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_WONT:
         printoption(data, "RCVD", CURL_WONT, c);
         tn->please_negotiate = 1;
         rec_wont(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_DO:
         printoption(data, "RCVD", CURL_DO, c);
         tn->please_negotiate = 1;
         rec_do(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_DONT:
         printoption(data, "RCVD", CURL_DONT, c);
         tn->please_negotiate = 1;
@@ -1000,7 +1029,7 @@ void telrcv(struct connectdata *conn,
             CURL_SB_ACCUM(tn, c);
             tn->subpointer -= 2;
             CURL_SB_TERM(tn);
-	    
+
             printoption(data, "In SUBOPTION processing, RCVD", CURL_IAC, c);
             suboption(conn);   /* handle sub-option */
             tn->telrcv_state = CURL_TS_IAC;
@@ -1023,9 +1052,11 @@ void telrcv(struct connectdata *conn,
   }
 }
 
-CURLcode Curl_telnet_done(struct connectdata *conn)
+CURLcode Curl_telnet_done(struct connectdata *conn, CURLcode status)
 {
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
+  (void)status; /* unused */
+
   curl_slist_free_all(tn->telnet_vars);
 
   free(conn->proto.telnet);
@@ -1038,20 +1069,28 @@ CURLcode Curl_telnet(struct connectdata *conn)
 {
   CURLcode code;
   struct SessionHandle *data = conn->data;
-  int sockfd = conn->firstsocket;
+  curl_socket_t sockfd = conn->sock[FIRSTSOCKET];
 #ifdef WIN32
+  HMODULE wsock2;
+  WSOCK2_FUNC close_event_func;
+  WSOCK2_FUNC create_event_func;
+  WSOCK2_FUNC event_select_func;
+  WSOCK2_FUNC enum_netevents_func;
   WSAEVENT event_handle;
   WSANETWORKEVENTS events;
   HANDLE stdin_handle;
   HANDLE objs[2];
+  DWORD  obj_count;
+  DWORD  wait_timeout;
   DWORD waitret;
+  DWORD readfile_read;
 #else
-  fd_set readfd;
-  fd_set keepfd;
+  int interval_ms;
+  struct pollfd pfd[2];
 #endif
+  ssize_t nread;
   bool keepon = TRUE;
   char *buf = data->state.buffer;
-  ssize_t nread;
   struct TELNET *tn;
 
   code = init_telnet(conn);
@@ -1065,112 +1104,227 @@ CURLcode Curl_telnet(struct connectdata *conn)
     return code;
 
 #ifdef WIN32
+  /*
+  ** This functionality only works with WinSock >= 2.0.  So,
+  ** make sure have it.
+  */
+  code = check_wsock2(data);
+  if (code)
+    return code;
+
+  /* OK, so we have WinSock 2.0.  We need to dynamically */
+  /* load ws2_32.dll and get the function pointers we need. */
+  wsock2 = LoadLibrary("WS2_32.DLL");
+  if (wsock2 == NULL) {
+    failf(data,"failed to load WS2_32.DLL (%d)",GetLastError());
+    return CURLE_FAILED_INIT;
+  }
+
+  /* Grab a pointer to WSACreateEvent */
+  create_event_func = GetProcAddress(wsock2,"WSACreateEvent");
+  if (create_event_func == NULL) {
+    failf(data,"failed to find WSACreateEvent function (%d)",
+          GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSACloseEvent */
+  close_event_func = GetProcAddress(wsock2,"WSACloseEvent");
+  if (create_event_func == NULL) {
+    failf(data,"failed to find WSACloseEvent function (%d)",
+          GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSAEventSelect */
+  event_select_func = GetProcAddress(wsock2,"WSAEventSelect");
+  if (event_select_func == NULL) {
+    failf(data,"failed to find WSAEventSelect function (%d)",
+          GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
+  /* And WSAEnumNetworkEvents */
+  enum_netevents_func = GetProcAddress(wsock2,"WSAEnumNetworkEvents");
+  if (enum_netevents_func == NULL) {
+    failf(data,"failed to find WSAEnumNetworkEvents function (%d)",
+          GetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
+
   /* We want to wait for both stdin and the socket. Since
   ** the select() function in winsock only works on sockets
   ** we have to use the WaitForMultipleObjects() call.
   */
 
   /* First, create a sockets event object */
-  event_handle = WSACreateEvent();
+  event_handle = (WSAEVENT)create_event_func();
+  if (event_handle == WSA_INVALID_EVENT) {
+    failf(data,"WSACreateEvent failed (%d)",WSAGetLastError());
+    FreeLibrary(wsock2);
+    return CURLE_FAILED_INIT;
+  }
 
   /* The get the Windows file handle for stdin */
   stdin_handle = GetStdHandle(STD_INPUT_HANDLE);
 
   /* Create the list of objects to wait for */
-  objs[0] = stdin_handle;
-  objs[1] = event_handle;
+  objs[0] = event_handle;
+  objs[1] = stdin_handle;
 
   /* Tell winsock what events we want to listen to */
-  if(WSAEventSelect(sockfd, event_handle, FD_READ|FD_CLOSE) == SOCKET_ERROR) {
+  if(event_select_func(sockfd, event_handle, FD_READ|FD_CLOSE) == SOCKET_ERROR) {
+    close_event_func(event_handle);
+    FreeLibrary(wsock2);
     return 0;
   }
 
+  /* If stdin_handle is a pipe, use PeekNamedPipe() method to check it,
+     else use the old WaitForMultipleObjects() way */
+  if(GetFileType(stdin_handle) == FILE_TYPE_PIPE) {
+    /* Don't wait for stdin_handle, just wait for event_handle */
+    obj_count = 1;
+    /* Check stdin_handle per 100 milliseconds */
+    wait_timeout = 100;
+  } else {
+    obj_count = 2;
+    wait_timeout = INFINITE;
+  }
+
   /* Keep on listening and act on events */
   while(keepon) {
-    waitret = WaitForMultipleObjects(2, objs, FALSE, INFINITE);
-    switch(waitret - WAIT_OBJECT_0)
+    waitret = WaitForMultipleObjects(obj_count, objs, FALSE, wait_timeout);
+    switch(waitret) {
+    case WAIT_TIMEOUT:
     {
-      case 0:
-      {
-        unsigned char outbuf[2];
-        int out_count = 0;
-        ssize_t bytes_written;
-        char *buffer = buf;
-              
-        if(!ReadFile(stdin_handle, buf, 255, (LPDWORD)&nread, NULL)) {
+      unsigned char outbuf[2];
+      int out_count = 0;
+      ssize_t bytes_written;
+      char *buffer = buf;
+
+      for(;;) {
+        if(!PeekNamedPipe(stdin_handle, NULL, 0, NULL, &readfile_read, NULL)) {
           keepon = FALSE;
           break;
         }
-        
+        nread = readfile_read;
+
+        if(!nread)
+          break;
+
+        if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
+                     &readfile_read, NULL)) {
+          keepon = FALSE;
+          break;
+        }
+        nread = readfile_read;
+
         while(nread--) {
           outbuf[0] = *buffer++;
           out_count = 1;
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
-          
-          Curl_write(conn, conn->firstsocket, outbuf,
+
+          Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
-      break;
-      
-      case 1:
-        if(WSAEnumNetworkEvents(sockfd, event_handle, &events)
-           != SOCKET_ERROR)
-        {
-          if(events.lNetworkEvents & FD_READ)
-          {
-            /* This reallu OUGHT to check its return code. */
-            Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
-            
-            telrcv(conn, (unsigned char *)buf, nread);
-            
-            fflush(stdout);
-            
-            /* Negotiate if the peer has started negotiating,
-               otherwise don't. We don't want to speak telnet with
-               non-telnet servers, like POP or SMTP. */
-            if(tn->please_negotiate && !tn->already_negotiated) {
-              negotiate(conn);
-              tn->already_negotiated = 1;
-            }
-          }
-          
-          if(events.lNetworkEvents & FD_CLOSE)
-          {
-            keepon = FALSE;
+    }
+    break;
+
+    case WAIT_OBJECT_0 + 1:
+    {
+      unsigned char outbuf[2];
+      int out_count = 0;
+      ssize_t bytes_written;
+      char *buffer = buf;
+
+      if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
+                   &readfile_read, NULL)) {
+        keepon = FALSE;
+        break;
+      }
+      nread = readfile_read;
+
+      while(nread--) {
+        outbuf[0] = *buffer++;
+        out_count = 1;
+        if(outbuf[0] == CURL_IAC)
+          outbuf[out_count++] = CURL_IAC;
+
+        Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
+                   out_count, &bytes_written);
+      }
+    }
+    break;
+
+    case WAIT_OBJECT_0:
+      if(enum_netevents_func(sockfd, event_handle, &events)
+         != SOCKET_ERROR) {
+        if(events.lNetworkEvents & FD_READ) {
+          /* This reallu OUGHT to check its return code. */
+          (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+
+          telrcv(conn, (unsigned char *)buf, nread);
+
+          fflush(stdout);
+
+          /* Negotiate if the peer has started negotiating,
+             otherwise don't. We don't want to speak telnet with
+             non-telnet servers, like POP or SMTP. */
+          if(tn->please_negotiate && !tn->already_negotiated) {
+            negotiate(conn);
+            tn->already_negotiated = 1;
           }
         }
-        break;
+
+        if(events.lNetworkEvents & FD_CLOSE) {
+          keepon = FALSE;
+        }
+      }
+      break;
     }
   }
-#else
-  FD_ZERO (&readfd);		/* clear it */
-  FD_SET (sockfd, &readfd);
-  FD_SET (0, &readfd);
 
-  keepfd = readfd;
+  /* We called WSACreateEvent, so call WSACloseEvent */
+  if (close_event_func(event_handle) == FALSE) {
+    infof(data,"WSACloseEvent failed (%d)",WSAGetLastError());
+  }
+
+  /* "Forget" pointers into the library we're about to free */
+  create_event_func = NULL;
+  close_event_func = NULL;
+  event_select_func = NULL;
+  enum_netevents_func = NULL;
+
+  /* We called LoadLibrary, so call FreeLibrary */
+  if (!FreeLibrary(wsock2))
+    infof(data,"FreeLibrary(wsock2) failed (%d)",GetLastError());
+#else
+  pfd[0].fd = sockfd;
+  pfd[0].events = POLLIN;
+  pfd[1].fd = 0;
+  pfd[1].events = POLLIN;
+  interval_ms = 1 * 1000;
 
   while (keepon) {
-    struct timeval interval;
-
-    readfd = keepfd;		/* set this every lap in the loop */
-    interval.tv_sec = 1;
-    interval.tv_usec = 0;
-
-    switch (select (sockfd + 1, &readfd, NULL, NULL, &interval)) {
-    case -1:			/* error, stop reading */
+    switch (Curl_poll(pfd, 2, interval_ms)) {
+    case -1:                    /* error, stop reading */
       keepon = FALSE;
       continue;
-    case 0:			/* timeout */
+    case 0:                     /* timeout */
       break;
-    default:			/* read! */
-      if(FD_ISSET(0, &readfd)) { /* read from stdin */
+    default:                    /* read! */
+      if(pfd[1].revents & POLLIN) { /* read from stdin */
         unsigned char outbuf[2];
         int out_count = 0;
         ssize_t bytes_written;
         char *buffer = buf;
-        
+
         nread = read(0, buf, 255);
 
         while(nread--) {
@@ -1178,15 +1332,15 @@ CURLcode Curl_telnet(struct connectdata *conn)
           out_count = 1;
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
-      
-          Curl_write(conn, conn->firstsocket, outbuf,
+
+          Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
 
-      if(FD_ISSET(sockfd, &readfd)) {
+      if(pfd[0].revents & POLLIN) {
         /* This OUGHT to check the return code... */
-        Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+        (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
 
         /* if we receive 0 or less here, the server closed the connection and
            we bail out from this! */
