@@ -1,10 +1,10 @@
 #ifndef __URLDATA_H
 #define __URLDATA_H
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
  * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
@@ -12,7 +12,7 @@
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: urldata.h,v 1.217 2004/04/22 20:07:41 bagder Exp $
+ * $Id: urldata.h,v 1.252 2004/12/19 09:37:32 bagder Exp $
  ***************************************************************************/
 
 /* This file is for lib internal stuff */
@@ -48,7 +48,7 @@
 
 #include "cookie.h"
 #include "formdata.h"
-    
+
 #ifdef USE_SSLEAY
 /* SSLeay stuff usually in /usr/local/ssl/include */
 #ifdef USE_OPENSSL
@@ -61,6 +61,9 @@
 #ifdef HAVE_OPENSSL_ENGINE_H
 #include <openssl/engine.h>
 #endif
+#ifdef HAVE_OPENSSL_PKCS12_H
+#include <openssl/pkcs12.h>
+#endif
 #else
 #include "rsa.h"
 #include "crypto.h"
@@ -78,7 +81,11 @@
 #include "timeval.h"
 
 #ifdef HAVE_ZLIB_H
-#include <zlib.h> 		/* for content-encoding */
+#include <zlib.h>               /* for content-encoding */
+#endif
+
+#ifdef USE_ARES
+#include <ares.h>
 #endif
 
 #include <curl/curl.h>
@@ -96,10 +103,6 @@
 #endif
 #endif
 
-#ifdef USE_ARES
-#include <ares.h>
-#endif
-
 /* Download buffer size, keep it fairly big for speed reasons */
 #define BUFSIZE CURL_MAX_WRITE_SIZE
 
@@ -119,11 +122,11 @@ struct krb4buffer {
   size_t index;
   int eof_flag;
 };
-enum protection_level { 
-    prot_clear, 
-    prot_safe, 
-    prot_confidential, 
-    prot_private 
+enum protection_level {
+    prot_clear,
+    prot_safe,
+    prot_confidential,
+    prot_private
 };
 #endif
 
@@ -151,8 +154,8 @@ struct ssl_config_data {
   char *egdsocket;       /* path to file containing the EGD daemon socket */
   char *cipher_list;     /* list of ciphers to use */
   long numsessions;      /* SSL session id cache size */
-  curl_ssl_ctx_callback fsslctx; 	/* function to initialize ssl ctx */
-  void *fsslctxp; 	/*parameter for call back */
+  curl_ssl_ctx_callback fsslctx;        /* function to initialize ssl ctx */
+  void *fsslctxp;       /*parameter for call back */
 };
 
 /* information stored about one single SSL session */
@@ -171,6 +174,10 @@ struct digestdata {
   char *realm;
   int algo;
   bool stale; /* set true for re-negotiation */
+  char *opaque;
+  char *qop;
+  char *algorithm;
+  int nc; /* nounce count */
 };
 
 typedef enum {
@@ -181,6 +188,12 @@ typedef enum {
   NTLMSTATE_LAST
 } curlntlm;
 
+/* for 3rd party transfers to decide which side that issues PASV */
+typedef enum {
+  CURL_TARGET_PASV,
+  CURL_SOURCE_PASV
+} curl_pasv_side;
+
 /* Struct used for NTLM challenge-response authentication */
 struct ntlmdata {
   curlntlm state;
@@ -208,7 +221,7 @@ struct HTTP {
 
   const char *p_pragma;      /* Pragma: string */
   const char *p_accept;      /* Accept: string */
-  curl_off_t readbytecount; 
+  curl_off_t readbytecount;
   curl_off_t writebytecount;
 
   /* For FORM posting */
@@ -258,17 +271,22 @@ struct FTP {
   long response_time; /* When no timeout is given, this is the amount of
                          seconds we await for an FTP response. Initialized
                          in Curl_ftp_connect() */
-  bool ctl_valid;     /* Tells Curl_ftp_quit() whether or not to do 
-                         anything. If the connection has timed out or
-                         been closed, this should be FALSE when it gets
-                         to Curl_ftp_quit() */
+  bool ctl_valid;   /* Tells Curl_ftp_quit() whether or not to do anything. If
+                       the connection has timed out or been closed, this
+                       should be FALSE when it gets to Curl_ftp_quit() */
+  bool cwddone;     /* if it has been determined that the proper CWD combo
+                       already has been done */
+  char *prevpath;   /* conn->path from the previous transfer */
 };
 
 /****************************************************************************
  * FILE unique setup
  ***************************************************************************/
 struct FILEPROTO {
-  int fd; /* open file descriptor to read from! */
+  char *path; /* the path we operate on */
+  char *freepath; /* pointer to the allocated block we must free, this might
+                     differ from the 'path' pointer */
+  int fd;     /* open file descriptor to read from! */
 };
 
 /*
@@ -302,6 +320,35 @@ struct ConnectBits {
   bool retry;         /* this connection is about to get closed and then
                          re-attempted at another connection. */
   bool no_body;       /* CURLOPT_NO_BODY (or similar) was set */
+  bool tunnel_proxy;  /* if CONNECT is used to "tunnel" through the proxy.
+                         This is implicit when SSL-protocols are used through
+                         proxies, but can also be enabled explicitly by
+                         apps */
+  bool authneg;       /* TRUE when the auth phase has started, which means
+                         that we are creating a request with an auth header,
+                         but it is not the final request in the auth
+                         negotiation. */
+  bool rewindaftersend;/* TRUE when the sending couldn't be stopped even
+                          though it will be discarded. When the whole send
+                          operation is done, we must call the data rewind
+                          callback. */
+  bool ftp_use_epsv;  /* As set with CURLOPT_FTP_USE_EPSV, but if we find out
+                         EPSV doesn't work we disable it for the forthcoming
+                         requests */
+
+  bool ftp_use_eprt;  /* As set with CURLOPT_FTP_USE_EPRT, but if we find out
+                         EPRT doesn't work we disable it for the forthcoming
+                         requests */
+  bool ftp_use_lprt;  /* As set with CURLOPT_FTP_USE_EPRT, but if we find out
+                         LPRT doesn't work we disable it for the forthcoming
+                         requests */
+};
+
+struct hostname {
+  char *rawalloc; /* allocated "raw" version of the name */
+  char *encalloc; /* allocated IDN-encoded version of the name */
+  char *name;     /* name to use internally, might be encoded, might be raw */
+  char *dispname; /* name to display, as 'name' might be encoded */
 };
 
 /*
@@ -311,31 +358,31 @@ struct ConnectBits {
  */
 
 struct Curl_transfer_keeper {
-  curl_off_t bytecount;                /* total number of bytes read */
-  int writebytecount;           /* number of bytes written */
+  curl_off_t bytecount;         /* total number of bytes read */
+  curl_off_t writebytecount;    /* number of bytes written */
   struct timeval start;         /* transfer started at this time */
   struct timeval now;           /* current time */
-  bool header;	                /* incoming data has HTTP header */
+  bool header;                  /* incoming data has HTTP header */
   enum {
-    HEADER_NORMAL,      /* no bad header at all */
-    HEADER_PARTHEADER,  /* part of the chunk is a bad header, the rest is
-                           normal data */
-    HEADER_ALLBAD       /* all was believed to be header */
-  } badheader;		        /* the header was deemed bad and will be
+    HEADER_NORMAL,              /* no bad header at all */
+    HEADER_PARTHEADER,          /* part of the chunk is a bad header, the rest
+                                   is normal data */
+    HEADER_ALLBAD               /* all was believed to be header */
+  } badheader;                  /* the header was deemed bad and will be
                                    written as body */
-  int headerline;		/* counts header lines to better track the
+  int headerline;               /* counts header lines to better track the
                                    first one */
-  char *hbufp;			/* points at *end* of header line */
-  int hbuflen;
-  char *str;			/* within buf */
-  char *str_start;		/* within buf */
-  char *end_ptr;		/* within buf */
-  char *p;			/* within headerbuff */
-  bool content_range;      	/* set TRUE if Content-Range: was found */
-  curl_off_t offset;	                /* possible resume offset read from the
+  char *hbufp;                  /* points at *end* of header line */
+  size_t hbuflen;
+  char *str;                    /* within buf */
+  char *str_start;              /* within buf */
+  char *end_ptr;                /* within buf */
+  char *p;                      /* within headerbuff */
+  bool content_range;           /* set TRUE if Content-Range: was found */
+  curl_off_t offset;            /* possible resume offset read from the
                                    Content-Range: header */
-  int httpcode;		        /* error code from the 'HTTP/1.? XXX' line */
-  int httpversion;		/* the HTTP version*10 */
+  int httpcode;                 /* error code from the 'HTTP/1.? XXX' line */
+  int httpversion;              /* the HTTP version*10 */
   struct timeval start100;      /* time stamp to wait for the 100 code from */
   bool write_after_100_header;  /* TRUE = we enable the write after we
                                    received a 100-continue/timeout or
@@ -343,20 +390,19 @@ struct Curl_transfer_keeper {
   bool wait100_after_headers;   /* TRUE = after the request-headers have been
                                    sent off properly, we go into the wait100
                                    state, FALSE = don't */
-  int content_encoding;  	/* What content encoding. sec 3.5, RFC2616. */
+  int content_encoding;         /* What content encoding. sec 3.5, RFC2616. */
 
-#define IDENTITY 0		/* No encoding */
-#define DEFLATE 1		/* zlib delfate [RFC 1950 & 1951] */
-#define GZIP 2			/* gzip algorithm [RFC 1952] */
-#define COMPRESS 3		/* Not handled, added for completeness */
+#define IDENTITY 0              /* No encoding */
+#define DEFLATE 1               /* zlib delfate [RFC 1950 & 1951] */
+#define GZIP 2                  /* gzip algorithm [RFC 1952] */
+#define COMPRESS 3              /* Not handled, added for completeness */
 
 #ifdef HAVE_LIBZ
-  bool zlib_init;		/* True if zlib already initialized;
-				   undefined if Content-Encoding header. */
-  z_stream z;			/* State structure for zlib. */
+  bool zlib_init;               /* True if zlib already initialized;
+                                   undefined if Content-Encoding header. */
+  z_stream z;                   /* State structure for zlib. */
 #endif
 
-  /* for the low speed checks: */
   time_t timeofdoc;
   long bodywrites;
 
@@ -381,7 +427,8 @@ struct Curl_transfer_keeper {
   bool ignorebody;  /* we read a response-body but we ignore it! */
 };
 
-#if defined(USE_ARES) || defined(USE_THREADING_GETHOSTBYNAME)
+#if defined(USE_ARES) || defined(USE_THREADING_GETHOSTBYNAME) || \
+    defined(USE_THREADING_GETADDRINFO)
 struct Curl_async {
   char *hostname;
   int port;
@@ -402,7 +449,7 @@ struct Curl_async {
 struct connectdata {
   /**** Fields set when inited and not modified again */
   struct SessionHandle *data; /* link to the root CURL struct */
-  int connectindex; /* what index in the connects index this particular
+  long connectindex; /* what index in the connects index this particular
                        struct has */
 
   long protocol; /* PROT_* flags concerning the protocol set */
@@ -419,17 +466,14 @@ struct connectdata {
 #define PROT_SSL     (1<<10) /* protocol requires SSL */
 
   /* the particular host we use, in two different ways */
-  struct Curl_dns_entry *connect_addr;
+  struct Curl_dns_entry *dns_entry;
+  Curl_addrinfo *ip_addr; /* the particular IP we connected to */
+
+  char protostr[16];  /* store the protocol string in this buffer */
+
+  struct hostname host;
+  struct hostname proxy;
 
-#ifdef ENABLE_IPV6
-  struct addrinfo *serv_addr;
-#else
-  struct sockaddr_in serv_addr;
-#endif
-  char protostr[64];  /* store the protocol string in this buffer */
-  char *namebuffer; /* allocated buffer to store the hostname in */
-  char *hostname;  /* hostname to use, as parsed from url. points to
-                      somewhere within the namebuffer[] area */
   char *pathbuffer;/* allocated buffer to store the URL's path part in */
   char *path;      /* path to use, points to somewhere within the pathbuffer
                       area */
@@ -448,21 +492,19 @@ struct connectdata {
                   this syntax. */
   curl_off_t resume_from; /* continue [ftp] transfer from here */
 
-  char *proxyhost; /* name of the http proxy host */
-
   char *user;    /* user name string, allocated */
   char *passwd;  /* password string, allocated */
 
   char *proxyuser;    /* proxy user name string, allocated */
   char *proxypasswd;  /* proxy password string, allocated */
-  
+
   struct timeval now;     /* "current" time */
   struct timeval created; /* creation time */
   curl_socket_t sock[2]; /* two sockets, the second is used for the data
                             transfer when doing FTP */
   curl_off_t maxdownload; /* in bytes, the maximum amount of data to fetch, 0
                              means unlimited */
-  
+
   struct ssl_connect_data ssl[2]; /* this is for ssl-stuff */
   struct ssl_config_data ssl_config;
 
@@ -471,7 +513,7 @@ struct connectdata {
   /* These two functions MUST be set by the curl_connect() function to be
      be protocol dependent */
   CURLcode (*curl_do)(struct connectdata *);
-  CURLcode (*curl_done)(struct connectdata *);
+  CURLcode (*curl_done)(struct connectdata *, CURLcode);
 
   /* If the curl_do() function is better made in two halves, this
    * curl_do_more() function will be called afterwards, if set. For example
@@ -481,26 +523,26 @@ struct connectdata {
 
   /* This function *MAY* be set to a protocol-dependent function that is run
    * after the connect() and everything is done, as a step in the connection.
-   */ 
+   */
   CURLcode (*curl_connect)(struct connectdata *);
 
   /* This function *MAY* be set to a protocol-dependent function that is run
    * by the curl_disconnect(), as a step in the disconnection.
-   */ 
+   */
   CURLcode (*curl_disconnect)(struct connectdata *);
 
   /* This function *MAY* be set to a protocol-dependent function that is run
    * in the curl_close() function if protocol-specific cleanups are required.
-   */ 
+   */
   CURLcode (*curl_close)(struct connectdata *);
 
   /**** curl_get() phase fields */
 
   /* READ stuff */
-  curl_socket_t sockfd;	  /* socket to read from or CURL_SOCKET_BAD */
-  curl_off_t size;	  /* -1 if unknown at this point */
+  curl_socket_t sockfd;   /* socket to read from or CURL_SOCKET_BAD */
+  curl_off_t size;        /* -1 if unknown at this point */
   curl_off_t *bytecountp; /* return number of bytes read or NULL */
-          
+
   /* WRITE stuff */
   curl_socket_t writesockfd; /* socket to write to, it may very
                                 well be the same we read from.
@@ -516,13 +558,12 @@ struct connectdata {
     char *userpwd; /* free later if not NULL! */
     char *rangeline; /* free later if not NULL! */
     char *ref; /* free later if not NULL! */
-    char *cookie; /* free later if not NULL! */
     char *host; /* free later if not NULL */
     char *cookiehost; /* free later if not NULL */
   } allocptr;
 
   char *newurl; /* This can only be set if a Location: was in the
-		   document headers */
+                   document headers */
 
 #ifdef HAVE_KRB4
   enum protection_level command_prot;
@@ -535,7 +576,7 @@ struct connectdata {
   int sec_complete;
   void *app_data;
 
-  struct Curl_sec_client_mech *mech;
+  const struct Curl_sec_client_mech *mech;
   struct sockaddr_in local_addr;
 
 #endif
@@ -549,10 +590,6 @@ struct connectdata {
     struct FTP *ftp;
     struct FILEPROTO *file;
     void *telnet;        /* private for telnet.c-eyes only */
-#if 0 /* no need for special ones for these: */
-    struct LDAP *ldap;
-    struct DICT *dict;
-#endif
     void *generic;
   } proto;
 
@@ -561,7 +598,7 @@ struct connectdata {
 
   /* 'upload_present' is used to keep a byte counter of how much data there is
      still left in the buffer, aimed for upload. */
-  int upload_present;
+  ssize_t upload_present;
 
    /* 'upload_fromhere' is used as a read-pointer when we uploaded parts of a
       buffer, so the next read should read from where this pointer points to,
@@ -579,10 +616,13 @@ struct connectdata {
 
   char syserr_buf [256]; /* buffer for Curl_strerror() */
 
-#if defined(USE_ARES) || defined(USE_THREADING_GETHOSTBYNAME)
+#if defined(USE_ARES) || defined(USE_THREADING_GETHOSTBYNAME) || \
+    defined(USE_THREADING_GETADDRINFO)
   /* data used for the asynch name resolve callback */
   struct Curl_async async;
 #endif
+  struct connectdata *sec_conn;   /* secondary connection for 3rd party
+                                     transfer */
 };
 
 /* The end of connectdata. */
@@ -604,6 +644,8 @@ struct PureInfo {
   long proxyauthavail;
   long httpauthavail;
 
+  long numconnects; /* how many new connection did libcurl created */
+
   char *contenttype; /* the content type of the object */
 };
 
@@ -666,6 +708,19 @@ typedef enum {
 #define MAX_CURL_USER_LENGTH_TXT "255"
 #define MAX_CURL_PASSWORD_LENGTH_TXT "255"
 
+struct auth {
+  long want;  /* Bitmask set to the authentication methods wanted by the app
+                 (with CURLOPT_HTTPAUTH or CURLOPT_PROXYAUTH). */
+  long picked;
+  long avail; /* bitmask for what the server reports to support for this
+                 resource */
+  bool done;  /* TRUE when the auth phase is done and ready to do the *actual*
+                 request */
+  bool multi; /* TRUE if this is not yet authenticated but within the auth
+                 multipass negotiation */
+
+};
+
 struct UrlState {
   enum {
     Curl_if_none,
@@ -682,7 +737,7 @@ struct UrlState {
   long numconnects; /* size of the 'connects' array */
 
   char *headerbuff; /* allocated buffer to store headers in */
-  int headersize;   /* size of the allocation */
+  size_t headersize;   /* size of the allocation */
 
   char buffer[BUFSIZE+1]; /* download buffer */
   char uploadbuffer[BUFSIZE+1]; /* upload buffer */
@@ -690,10 +745,10 @@ struct UrlState {
                                 bytes / second */
   bool this_is_a_follow; /* this is a followed Location: request */
 
-  char *auth_host; /* if set, this should be the host name that we will
-                      sent authorization to, no else. Used to make Location:
-                      following not keep sending user+password... This is
-                      strdup() data.
+  char *first_host; /* if set, this should be the host name that we will
+                       sent authorization to, no else. Used to make Location:
+                       following not keep sending user+password... This is
+                       strdup() data.
                     */
 
   struct curl_ssl_session *session; /* array of 'numsessions' size */
@@ -703,7 +758,7 @@ struct UrlState {
   bool errorbuf; /* Set to TRUE if the error buffer is already filled in.
                     This must be set to FALSE every time _easy_perform() is
                     called. */
-
+  int os_errno;  /* filled in with errno whenever an error occurs */
 #ifdef HAVE_SIGNAL
   /* storage for the previous bag^H^H^HSIGPIPE signal handler :-) */
   void (*prev_signal)(int sig);
@@ -712,25 +767,23 @@ struct UrlState {
                       is always set TRUE when curl_easy_perform() is called. */
 
   struct digestdata digest;
+  struct digestdata proxydigest;
 
 #ifdef HAVE_GSSAPI
   struct negotiatedata negotiate;
 #endif
 
-  long authstage; /*   0 - authwant and authavail are still not initialized
-                     401 - web authentication is performed
-                     407 - proxy authentication is performed */
-  long authwant;  /* initially set to authentication methods requested by
-                     client (either with CURLOPT_HTTPAUTH or CURLOPT_PROXYAUTH
-                     depending on authstage) */
-  long authavail; /* what the server reports */
+  struct auth authhost;
+  struct auth authproxy;
 
   bool authproblem; /* TRUE if there's some problem authenticating */
-  bool authdone; /* TRUE when the auth phase is done and ready
-                    to do the *actual* request */
 #ifdef USE_ARES
   ares_channel areschannel; /* for name resolves */
 #endif
+
+#if defined(USE_SSLEAY) && defined(HAVE_OPENSSL_ENGINE_H)
+  ENGINE *engine;
+#endif /* USE_SSLEAY */
 };
 
 
@@ -746,7 +799,7 @@ struct DynamicStatic {
   bool url_alloc;   /* URL string is malloc()'ed */
   bool url_changed; /* set on CURL_OPT_URL, used to detect if the URL was
                        changed after the connect phase, as we allow callback
-                       to change it and if so, we reconnect to use the new 
+                       to change it and if so, we reconnect to use the new
                        URL instead */
   char *proxy;      /* work proxy, copied from UserDefined */
   bool proxy_alloc; /* http proxy string is malloc()'ed */
@@ -771,7 +824,7 @@ struct UserDefined {
   char *proxyuserpwd;  /* Proxy <user:password>, if used */
   long proxyport; /* If non-zero, use this port number by default. If the
                      proxy string features a ":[port]" that one will override
-                     this. */  
+                     this. */
   void *out;         /* the fetched file goes here */
   void *in;          /* the uploaded file is read from here */
   void *writeheader; /* write the header to this is non-NULL */
@@ -801,7 +854,9 @@ struct UserDefined {
   curl_read_callback fread;          /* function that reads the input */
   curl_progress_callback fprogress;  /* function for progress information */
   curl_debug_callback fdebug;      /* function that write informational data */
+  curl_ioctl_callback ioctl;       /* function for I/O control */
   void *progress_client; /* pointer to pass to the progress callback */
+  void *ioctl_client;   /* pointer to pass to the ioctl callback */
   long timeout;         /* in seconds, 0 means no timeout */
   long connecttimeout;  /* in seconds, 0 means no timeout */
   long ftp_response_timeout; /* in seconds, 0 means no timeout */
@@ -817,13 +872,16 @@ struct UserDefined {
   char *key;            /* private key */
   char *key_type;       /* format for private key (default: PEM) */
   char *key_passwd;     /* plain text private key password */
-  char *crypto_engine;  /* name of the crypto engine to use */
   char *cookiejar;      /* dump all cookies to this file */
   bool cookiesession;   /* new cookie session? */
   bool crlf;            /* convert crlf on ftp upload(?) */
   struct curl_slist *quote;     /* after connection is established */
   struct curl_slist *postquote; /* after the transfer */
-  struct curl_slist *prequote; /* before the transfer, after type (Wesley Laxton)*/
+  struct curl_slist *prequote; /* before the transfer, after type */
+  struct curl_slist *source_prequote;  /* in 3rd party transfer mode - before
+                                          the transfer on source host */
+  struct curl_slist *source_postquote; /* in 3rd party transfer mode - after
+                                          the transfer on source host */
   struct curl_slist *telnet_options; /* linked list of telnet options */
   curl_TimeCond timecondition; /* kind of time/date comparison */
   time_t timevalue;       /* what time to compare with */
@@ -844,18 +902,25 @@ struct UserDefined {
   int dns_cache_timeout; /* DNS cache timeout */
   long buffer_size;      /* size of receive buffer to use */
 
-  char *private; /* Private data */
+  char *private_data; /* Private data */
 
   struct curl_slist *http200aliases; /* linked list of aliases for http200 */
 
-  int ip_version; 
+  long ip_version;
 
   curl_off_t max_filesize; /* Maximum file size to download */
-  
+
+  char *source_host;     /* for 3rd party transfer */
+  char *source_port;     /* for 3rd party transfer */
+  char *source_userpwd;  /* for 3rd party transfer */
+  char *source_path;     /* for 3rd party transfer */
+  curl_pasv_side pasvHost; /* for 3rd party transfer indicates passive host */
+
 /* Here follows boolean settings that define how to behave during
    this session. They are STATIC, set by libcurl users or at least initially
    and they don't change during operations. */
 
+  bool printhost;       /* printing host name in debug info */
   bool get_filetime;
   bool tunnel_thru_httpproxy;
   bool ftp_append;
@@ -884,7 +949,9 @@ struct UserDefined {
   bool expect100header;  /* TRUE if we added Expect: 100-continue */
   bool ftp_use_epsv;     /* if EPSV is to be attempted or not */
   bool ftp_use_eprt;     /* if EPRT is to be attempted or not */
+  bool ftp_use_lprt;     /* if LPRT is to be attempted or not */
   curl_ftpssl ftp_ssl;   /* if AUTH TLS is to be attempted etc */
+  curl_ftpauth ftpsslauth; /* what AUTH XXX to be attempted */
   bool no_signal;        /* do not use any signal/alarm handler */
   bool global_dns_cache; /* subject for future removal */
   bool tcp_nodelay;      /* whether to enable TCP_NODELAY or not */
@@ -913,9 +980,6 @@ struct SessionHandle {
   struct UrlState state;       /* struct for fields used for state info and
                                   other dynamic purposes */
   struct PureInfo info;        /* stats, reports and info data */
-#if defined(USE_SSLEAY) && defined(HAVE_OPENSSL_ENGINE_H)
-  ENGINE*  engine;
-#endif /* USE_SSLEAY */
 };
 
 #define LIBCURL_NAME "libcurl"
