@@ -1,18 +1,18 @@
 #ifndef __CLIENT_SETUP_H
 #define __CLIENT_SETUP_H
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -20,10 +20,10 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.16 2003/01/16 21:08:13 bagder Exp $
+ * $Id: setup.h,v 1.33 2004/12/15 14:05:07 bagder Exp $
  ***************************************************************************/
 
-#include <stdio.h>
+#define CURL_NO_OLDIES
 
 #if !defined(WIN32) && defined(__WIN32__)
 /* Borland fix */
@@ -31,11 +31,7 @@
 #endif
 
 #ifdef HAVE_CONFIG_H
-#ifdef VMS
-#include "config-vms.h"
-#else
 #include "config.h" /* the configure script results */
-#endif
 #else
 #ifdef WIN32
 /* include the hand-modified win32 adjusted config.h! */
@@ -48,6 +44,34 @@
 #ifdef __riscos__
 #include "config-riscos.h"
 #endif
+#ifdef __amigaos__
+#include "config-amigaos.h"
+#endif
+#endif
+
+#ifdef CURLDEBUG
+/* This is an ugly hack for CURLDEBUG conditions only. We need to include
+   the file here, since it might set the _FILE_OFFSET_BITS define, which must
+   be set BEFORE all normal system headers. */
+#include "../lib/setup.h"
+#endif
+
+#include <stdio.h>
+
+#ifdef __TANDEM
+#include <floss.h>
+#endif
+
+#ifndef TRUE
+#define TRUE 1
+#endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+
+#if !defined(__cplusplus) && !defined(__BEOS__) && !defined(typedef_bool)
+typedef char bool;
+#define typedef_bool
 #endif
 
 #ifndef OS
@@ -59,7 +83,6 @@ int fileno( FILE *stream);
 #endif
 
 #ifdef WIN32
-#define PATH_CHAR     ";"
 #define DIR_CHAR      "\\"
 #define DOT_CHAR      "_"
 #else
@@ -67,20 +90,37 @@ int fileno( FILE *stream);
 /* 20000318 mgs
  * OS/2 supports leading dots in filenames if the volume is formatted
  * with JFS or HPFS. */
-#define PATH_CHAR     ";"
 #define DIR_CHAR      "\\"
 #define DOT_CHAR      "."
 #else
 
-#define PATH_CHAR     ":"
+#ifdef DJGPP
+#include <tcp.h>
+#ifdef word
+#undef word
+#endif
+#define HAVE_LIMITS_H /* we have limits.h */
+#define DIR_CHAR      "/"
+#define DOT_CHAR      "_"
+#else
+
 #define DIR_CHAR      "/"
 #define DOT_CHAR      "."
 
-#endif
-#endif
+#endif /* !DJGPP */
+#endif /* !__EMX__ */
+#endif /* !WIN32 */
 
 #ifdef __riscos__
 #define USE_ENVIRONMENT
 #endif
 
+#ifdef __BEOS__
+#define typedef_bool
+#endif
+
+#ifndef SIZEOF_CURL_OFF_T
+#define SIZEOF_CURL_OFF_T sizeof(curl_off_t)
+#endif
+
 #endif /* __SETUP_H */
