@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: ftpserver.pl,v 1.42 2004/04/19 15:20:30 bagder Exp $
+# $Id: ftpserver.pl,v 1.47 2004/10/27 14:18:57 bagder Exp $
 ###########################################################################
 
 # This is the FTP server designed for the curl test suite.
@@ -43,8 +43,7 @@ sub logmsg {
     my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
         localtime(time);
     open(FTPLOG, ">>log/ftpd.log");
-    printf FTPLOG ("%02d:%02d:%02d (%d) ",
-                   $hour, $min, $sec, $$);
+    printf FTPLOG ("%02d:%02d:%02d ", $hour, $min, $sec);
     print FTPLOG @_;
     close(FTPLOG);
 }
@@ -67,6 +66,7 @@ my $verbose=0; # set to 1 for debugging
 my $retrweirdo=0;
 my $retrnosize=0;
 my $srcdir=".";
+my $nosave=0;
 
 my $port = 8921; # just a default
 do {
@@ -318,7 +318,9 @@ sub RETR_command {
         $size += length($_);
     }
 
-    if($size) {
+    my %hash = getpartattr("reply", "data");
+
+    if($size || $hash{'sendzero'}) {
     
         if($rest) {
             # move read pointer forward
@@ -382,7 +384,10 @@ sub STOR_command {
     my $ulsize=0;
     while (defined($line = <SOCK>)) {
         $ulsize += length($line);
-        print FILE $line;
+        print FILE $line if(!$nosave);
+    }
+    if($nosave) {
+        print FILE "$ulsize bytes would've been stored here\n";
     }
     close(FILE);
     close_dataconn();
@@ -433,13 +438,32 @@ sub PASV_command {
         printf("229 Entering Passive Mode (|||%d|)\n", $pasvport);
     }
 
-    my $paddr = accept(SOCK, Server2);
-    my($iport,$iaddr) = sockaddr_in($paddr);
-    my $name = gethostbyaddr($iaddr,AF_INET);
 
-    close(Server2); # close the listener when its served its purpose!
+    my $paddr;
+    eval {
+        local $SIG{ALRM} = sub { die "alarm\n" };
+        alarm 2; # assume swift operations!
+        $paddr = accept(SOCK, Server2);
+        alarm 0;
+    };
+    if ($@) {
+        # timed out
+        
+        close(Server2);
+        logmsg "accept failed\n";
+        return;
+    }
+    else {
+        logmsg "accept worked\n";
 
-    logmsg "data connection from $name [", inet_ntoa($iaddr), "] at port $iport\n";
+        my($iport,$iaddr) = sockaddr_in($paddr);
+        my $name = gethostbyaddr($iaddr,AF_INET);
+
+        close(Server2); # close the listener when its served its purpose!
+
+        logmsg "data connection from $name [", inet_ntoa($iaddr),
+        "] at port $iport\n";
+    }
 
     return;
 }
@@ -478,6 +502,9 @@ my %customcount;
 my %delayreply;
 sub customize {
     undef %customreply;
+
+    $nosave = 0; # default is to save as normal
+
     open(CUSTOM, "<log/ftpserver.cmd") ||
         return 1;
 
@@ -486,23 +513,32 @@ sub customize {
     while(<CUSTOM>) {
         if($_ =~ /REPLY ([A-Z]+) (.*)/) {
             $customreply{$1}=$2;
+            logmsg "FTPD: set custom reply for $1\n";
         }
         if($_ =~ /COUNT ([A-Z]+) (.*)/) {
             # we blank the customreply for this command when having
             # been used this number of times
             $customcount{$1}=$2;
+            logmsg "FTPD: blank custom reply for $1 after $2 uses\n";
         }
         elsif($_ =~ /DELAY ([A-Z]+) (\d*)/) {
             $delayreply{$1}=$2;
+            logmsg "FTPD: delay reply for $1 with $2 seconds\n";
         }
         elsif($_ =~ /RETRWEIRDO/) {
-            print "instructed to use RETRWEIRDO\n";
+            logmsg "FTPD: instructed to use RETRWEIRDO\n";
             $retrweirdo=1;
         }
         elsif($_ =~ /RETRNOSIZE/) {
-            print "instructed to use RETRNOSIZE\n";
+            logmsg "FTPD: instructed to use RETRNOSIZE\n";
             $retrnosize=1;
         }
+        elsif($_ =~ /NOSAVE/) {
+            # don't actually store the file we upload - to be used when
+            # uploading insanely huge amounts
+            $nosave = 1;
+            logmsg "FTPD: NOSAVE prevents saving of uploaded data\n";
+        }
     }
     close(CUSTOM);
 }
@@ -616,5 +652,6 @@ for ( $waitedpid = 0;
         }
             
     } # while(1)
+    logmsg "client disconnected\n";
     close(Client);
 }
