@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: runtests.pl,v 1.118 2004/04/23 08:38:43 bagder Exp $
+# $Id: runtests.pl,v 1.155 2004/12/17 10:09:06 bagder Exp $
 ###########################################################################
 # These should be the only variables that might be needed to get edited:
 
@@ -32,10 +32,16 @@ require "getpart.pm"; # array functions
 
 my $srcdir = $ENV{'srcdir'} || '.';
 my $HOSTIP="127.0.0.1";
-my $HOSTPORT=8999; # bad name, but this is the HTTP server port
-my $HTTPSPORT=8433; # this is the HTTPS server port
-my $FTPPORT=8921;  # this is the FTP server port
-my $FTPSPORT=8821;  # this is the FTPS server port
+my $HOST6IP="[::1]";
+
+my $base = 8990; # base port number
+
+my $HTTPPORT; # HTTP server port
+my $HTTP6PORT; # HTTP IPv6 server port
+my $HTTPSPORT; # HTTPS server port
+my $FTPPORT; # FTP server port
+my $FTPSPORT; # FTPS server port
+
 my $CURL="../src/curl"; # what curl executable to run on the tests
 my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
 my $LOGDIR="log";
@@ -57,6 +63,7 @@ my $TESTCASES="all";
 #
 
 my $HTTPPIDFILE=".http.pid";
+my $HTTP6PIDFILE=".http6.pid";
 my $HTTPSPIDFILE=".https.pid";
 my $FTPPIDFILE=".ftp.pid";
 my $FTPSPIDFILE=".ftps.pid";
@@ -76,8 +83,25 @@ my $memanalyze="./memanalyze.pl";
 my $stunnel = checkcmd("stunnel");
 my $valgrind = checkcmd("valgrind");
 
+my $valgrind_tool;
+if($valgrind) {
+    # since valgrind 2.1.x, '--tool' option is mandatory
+    # use it, if it is supported by the version installed on the system
+    system("valgrind --help 2>&1 | grep -- --tool > /dev/null 2>&1");
+    if (($? >> 8)==0) {
+        $valgrind_tool="--tool=memcheck ";
+    }
+}
+
+my $gdb = checkcmd("gdb");
+
 my $ssl_version; # set if libcurl is built with SSL support
 my $large_file;  # set if libcurl is built with large file support
+my $has_idn;     # set if libcurl is built with IDN support
+my $http_ipv6;   # set if HTTP server has IPv6 support
+my $has_ipv6;    # set if libcurl is built with IPv6 support
+my $has_libz;    # set if libcurl is built with libz support
+my $has_getrlimit;  # set if system has getrlimit()
 
 my $skipped=0;  # number of tests skipped; reported in main loop
 my %skipped;    # skipped{reason}=counter, reasons for skip
@@ -153,117 +177,106 @@ sub serverpid {
 # Memory allocation test and failure torture testing.
 #
 sub torture {
-    # start all test servers (http, https, ftp, ftps)
-    &startservers(("http", "https", "ftp", "ftps"));
-    my $c;
+    my $testcmd = shift;
+    my $gdbline = shift;
 
-    my @test=('http://%HOSTIP:%HOSTPORT/1',
-              'ftp://%HOSTIP:%FTPPORT/');
-    
-    # loop over the different tests commands
-    for(@test) {
-        my $cmdargs = "$_";
+    # remove memdump first to be sure we get a new nice and clean one
+    unlink($memdump);
 
-        $c++;
+    # First get URL from test server, ignore the output/result
+    system($testcmd);
 
-        if($tortnum && ($tortnum != $c)) {
-            next;
+    print " CMD: $testcmd\n" if($verbose);
+
+    # memanalyze -v is our friend, get the number of allocations made
+    my $count=0;
+    my @out = `$memanalyze -v $memdump`;
+    for(@out) {
+        if(/^Allocations: (\d+)/) {
+            $count = $1;
+            last;
         }
-        print "We want test $c\n";
-
-        my $redir=">log/torture.stdout 2>log/torture.stderr";
-
-        subVariables(\$cmdargs);
-
-        my $testcmd = "$CURL $cmdargs $redir";
-
-        # First get URL from test server, ignore the output/result
-        system($testcmd);
-
-        # Set up gdb-stuff if desired
-        if($gdbthis) {
-            open(GDBCMD, ">log/gdbcmd");
-            print GDBCMD "set args $cmdargs\n";
-            print GDBCMD "show args\n";
-            close(GDBCMD);
-            $testcmd = "gdb $CURL -x log/gdbcmd";
-        }
-
-        print "Torture test $c:\n";
-        print " CMD: $testcmd\n" if($verbose);
-        
-        # memanalyze -v is our friend, get the number of allocations made
-        my $count;
-        my @out = `$memanalyze -v $memdump`;
-        for(@out) {
-            if(/^Allocations: (\d+)/) {
-                $count = $1;
-                last;
-            }
-        }
-        if(!$count) {
-            # hm, no allocations in this fetch, ignore and get next
-            print "BEEEP, no allocs found for test $c!!!\n";
-            next;
-        }
-        print " $count allocations to excersize\n";
-
-        for ( 1 .. $count ) {
-            my $limit = $_;
-            my $fail;
-
-            if($tortalloc && ($tortalloc != $limit)) {
-                next;
-            }
-
-            print "Alloc no: $limit\r" if(!$gdbthis);
-            
-            # make the memory allocation function number $limit return failure
-            $ENV{'CURL_MEMLIMIT'} = $limit;
-
-            # remove memdump first to be sure we get a new nice and clean one
-            unlink($memdump);
-            
-            print "**> Alloc number $limit is now set to fail <**\n" if($gdbthis);
-
-            my $ret = system($testcmd);
-
-            # verify that it returns a proper error code, doesn't leak memory
-            # and doesn't core dump
-            if($ret & 255) {
-                print " system() returned $ret\n";
-                $fail=1;
-            }
-            else {
-                my @memdata=`$memanalyze $memdump`;
-                my $leak=0;
-                for(@memdata) {
-                    if($_ ne "") {
-                        # well it could be other memory problems as well, but
-                        # we call it leak for short here
-                        $leak=1;
-                    }
-                }
-                if($leak) {
-                    print "** MEMORY FAILURE\n";
-                    print @memdata;
-                    print `$memanalyze -l $memdump`;
-                    $fail = 1;
-                }
-            }
-            if($fail) {
-                print " Failed on alloc number $limit in test $c.\n",
-                " invoke with -t$c,$limit to repeat this single case.\n";
-                stopservers();
-                exit 1;
-            }
-        }
-        print "\n torture test $c did GOOD\n";
-
-        # all is well, now test a different kind of URL
     }
-    stopservers();
-    exit; # for now, we stop after these tests
+    if(!$count) {
+        print " found no allocs to make fail\n";
+        return 0;
+    }
+
+    print " $count allocations to make fail\n";
+
+    for ( 1 .. $count ) {
+        my $limit = $_;
+        my $fail;
+        my $dumped_core;
+
+        if($tortalloc && ($tortalloc != $limit)) {
+            next;
+        }
+
+        print "Fail alloc no: $limit\r" if($verbose);
+
+        # make the memory allocation function number $limit return failure
+        $ENV{'CURL_MEMLIMIT'} = $limit;
+
+        # remove memdump first to be sure we get a new nice and clean one
+        unlink($memdump);
+
+        print "**> Alloc number $limit is now set to fail <**\n" if($gdbthis);
+
+        my $ret;
+        if($gdbthis) {
+            system($gdbline)
+        }
+        else {
+            $ret = system($testcmd);
+        }
+
+        # Now clear the variable again
+        $ENV{'CURL_MEMLIMIT'} = undef;
+
+        if(-r "core") {
+            # there's core file present now!
+            print " core dumped!\n";
+            $dumped_core = 1;
+            $fail = 2;
+        }
+
+        # verify that it returns a proper error code, doesn't leak memory
+        # and doesn't core dump
+        if($ret & 255) {
+            print " system() returned $ret\n";
+            $fail=1;
+        }
+        else {
+            my @memdata=`$memanalyze $memdump`;
+            my $leak=0;
+            for(@memdata) {
+                if($_ ne "") {
+                    # well it could be other memory problems as well, but
+                    # we call it leak for short here
+                    $leak=1;
+                }
+            }
+            if($leak) {
+                print "** MEMORY FAILURE\n";
+                print @memdata;
+                print `$memanalyze -l $memdump`;
+                $fail = 1;
+            }
+        }
+        if($fail) {
+            print " Failed on alloc number $limit in test.\n",
+            " invoke with -t$limit to repeat this single case.\n";
+            stopservers();
+            exit 1;
+        }
+    }
+
+    print "torture OK\n";
+    return 0;
+
+    #stopservers();
+    #exit; # for now, we stop after these tests
 }
 
 #######################################################################
@@ -316,22 +329,44 @@ sub checkserver {
 # test server on the test-port!
 #
 sub runhttpserver {
-    my $verbose = $_[0];
+    my ($verbose, $ipv6) = @_;
     my $RUNNING;
     my $pid;
 
-    $pid = checkserver ($HTTPPIDFILE);
+    my $pidfile = $HTTPPIDFILE;
+    my $port = $HTTPPORT;
+    my $ip = $HOSTIP;
+    my $nameext;
+
+    if($ipv6) {
+        # if IPv6, use a different setup
+        $pidfile = $HTTP6PIDFILE;
+        $port = $HTTP6PORT;
+        $ip = $HOST6IP;
+        $nameext="-ipv6";
+    }
+
+    $pid = checkserver($pidfile);
 
     # verify if our/any server is running on this port
-    my $cmd = "$CURL -o log/verifiedserver --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null";
+    my $cmd = "$CURL -o log/verifiedserver -g \"http://$ip:$port/verifiedserver\" 2>log/verifystderr";
     print "CMD; $cmd\n" if ($verbose);
     my $res = system($cmd);
 
     $res >>= 8; # rotate the result
     my $data;
 
-    print "RUN: curl command returned $res\n" if ($verbose);
-
+    if($res && $verbose) {
+        open(ERR, "<log/verifystderr");
+        my @e = <ERR>;
+        close(ERR);
+        print "RUN: curl command returned $res\n";
+        for(@e) {
+            if($_ !~ /^([ \t]*)$/) {
+                print "RUN: $_";
+            }
+        }
+    }
     open(FILE, "<log/verifiedserver");
     my @file=<FILE>;
     close(FILE);
@@ -341,15 +376,15 @@ sub runhttpserver {
         $pid = 0+$1;
     }
     elsif($data || ($res != 7)) {
-        print "RUN: Unknown HTTP server is running on port $HOSTPORT\n";
+        print "RUN: Unknown server is running on port $port\n";
         return -2;
     }
 
     if($pid > 0) {
         my $res = kill (9, $pid); # die!
         if(!$res) {
-            print "RUN: Failed to kill test HTTP server, do it manually and",
-            " restart the tests.\n";
+            print "RUN: Failed to kill test HTTP$nameext server, do it ",
+            "manually and restart the tests.\n";
             exit;
         }
         sleep(1);
@@ -360,7 +395,7 @@ sub runhttpserver {
     if($dir) {
         $flag .= "-d \"$dir\" ";
     }
-    $cmd="$perl $srcdir/httpserver.pl $flag $HOSTPORT &";
+    $cmd="$perl $srcdir/httpserver.pl -p $pidfile $flag $port $ipv6 &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
@@ -369,7 +404,7 @@ sub runhttpserver {
     my $verified;
     for(1 .. 10) {
         # verify that our server is up and running:
-        my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
+        my $data=`$CURL --silent -g \"$ip:$port/verifiedserver\" 2>/dev/null`;
 
         if ( $data =~ /WE ROOLZ: (\d+)/ ) {
             $pid = 0+$1;
@@ -378,19 +413,19 @@ sub runhttpserver {
         }
         else {
             if($verbose) {
-                print STDERR "RUN: Retrying HTTP server existence in 3 sec\n";
+                print STDERR "RUN: Retrying HTTP$nameext server existence in 3 sec\n";
             }
             sleep(3);
             next;
         }
     }
     if(!$verified) {
-        print STDERR "RUN: failed to start our HTTP server\n";
+        print STDERR "RUN: failed to start our HTTP$nameext server\n";
         return -1;
     }
 
     if($verbose) {
-        print "RUN: HTTP server is now verified to be our server\n";
+        print "RUN: HTTP$nameext server is now verified to be our server\n";
     }
 
     return $pid;
@@ -419,7 +454,7 @@ sub runhttpsserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/httpsserver.pl $flag -s \"$stunnel\" -d $srcdir -r $HOSTPORT $HTTPSPORT &";
+    my $cmd="$perl $srcdir/httpsserver.pl $flag -s \"$stunnel\" -d $srcdir -r $HTTPPORT $HTTPSPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
@@ -457,25 +492,25 @@ sub runftpserver {
         print "RUN: Check port $FTPPORT for our own FTP server\n"
             if ($verbose);
 
-
         my $time=time();
         # check if this is our server running on this port:
-        my $data=`$CURL -m4 --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+        my @data=`$CURL -m4 --silent ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
+        my $line;
 
         # if this took more than 2 secs, we assume it "hung" on a weird server
         my $took = time()-$time;
-        
-        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
-            # this is our test server with a known pid!
-            $pid = 0+$1;
-        }
-        else {
-            if($data || ($took > 2)) {
-                # this is not a known server
-                print "RUN: Unknown server on our favourite port: $FTPPORT\n";
-                return -1;
+
+        foreach $line (@data) {
+            if ( $line =~ /WE ROOLZ: (\d+)/ ) {
+                # this is our test server with a known pid!
+                $pid = 0+$1;
             }
         }
+        if(!$pid && $data[0]) {
+            # this is not a known server
+            print "RUN: Unknown server on our favourite FTP port: $FTPPORT\n";
+            return -1;
+        }
     }
 
     if($pid > 0) {
@@ -488,7 +523,7 @@ sub runftpserver {
         }
         sleep(1);
     }
-    
+
     # now (re-)start our server:
     my $flag=$debugprotocol?"-v ":"";
     $flag .= "-s \"$srcdir\"";
@@ -499,22 +534,30 @@ sub runftpserver {
     system($cmd);
 
     my $verified;
+    $pid = 0;
     for(1 .. 10) {
         # verify that our server is up and running:
-        my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
-
-        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
-            $pid = 0+$1;
-            $verified = 1;
-            last;
+        my $line;
+        my $cmd="$CURL --silent ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null";
+        print "$cmd\n" if($verbose);
+        my @data = `$cmd`;
+        foreach $line (@data) {
+            if ( $line =~ /WE ROOLZ: (\d+)/ ) {
+                $pid = 0+$1;
+                $verified = 1;
+                last;
+            }
         }
-        else {
+        if(!$pid) {
             if($verbose) {
                 print STDERR "RUN: Retrying FTP server existence in 3 sec\n";
             }
             sleep(3);
             next;
         }
+        else {
+            last;
+        }
     }
     if(!$verified) {
         warn "RUN: failed to start our FTP server\n";
@@ -618,7 +661,7 @@ sub filteroff {
         print OUT $_;
     }
     close(IN);
-    close(OUT);    
+    close(OUT);
     return 0;
 }
 
@@ -636,7 +679,7 @@ sub compare {
     if($result) {
         if(!$short) {
             print "\n $subject FAILED:\n";
-            print showdiff($firstref, $secondref);
+            print showdiff($LOGDIR, $firstref, $secondref);
         }
         else {
             print "FAILED\n";
@@ -652,9 +695,10 @@ sub checkcurl {
 
     unlink($memdump); # remove this if there was one left
 
+    my $feat;
     my $curl;
     my $libcurl;
-    my @version=`$CURL -V 2>/dev/null`;
+    my @version=`$CURL --version 2>/dev/null`;
     for(@version) {
         chomp;
 
@@ -724,7 +768,7 @@ sub checkcurl {
             # at this point
         }
         elsif($_ =~ /^Features: (.*)/i) {
-            my $feat = $1;
+            $feat = $1;
             if($feat =~ /debug/i) {
                 # debug is a listed "feature", use that knowledge
                 $curl_debug = 1;
@@ -739,10 +783,43 @@ sub checkcurl {
                 # large file support
                 $large_file=1;
             }
+            if($feat =~ /IDN/i) {
+                # IDN support
+                $has_idn=1;
+            }
+            if($feat =~ /IPv6/i) {
+                $has_ipv6 = 1;
+            }
+            if($feat =~ /libz/i) {
+                $has_libz = 1;
+            }
         }
     }
     if(!$curl) {
-        die "couldn't run curl!"
+        die "couldn't run '$CURL'"
+    }
+
+    if(-r "../lib/config.h") {
+        open(CONF, "<../lib/config.h");
+        while(<CONF>) {
+            if($_ =~ /^\#define HAVE_GETRLIMIT/) {
+                $has_getrlimit = 1;
+            }
+        }
+        close(CONF);
+    }
+
+    if($has_ipv6) {
+        # client has ipv6 support, check that the HTTP server has it!
+        my @sws = `server/sws --version`;
+        if($sws[0] =~ /IPv6/) {
+            # HTTP server has ipv6 support!
+            $http_ipv6 = 1;
+        }
+    }
+
+    if(!$curl_debug && $torture) {
+        die "can't run torture tests since curl was not build with debug";
     }
 
     my $hostname=`hostname`;
@@ -751,25 +828,38 @@ sub checkcurl {
     print "********* System characteristics ******** \n",
     "* $curl\n",
     "* $libcurl\n",
+    "* Features: $feat\n",
     "* Host: $hostname",
     "* System: $hosttype";
 
-    printf("* Server SSL:       %s\n", $stunnel?"ON":"OFF");
-    printf("* libcurl SSL:      %s\n", $ssl_version?"ON":"OFF");
-    printf("* libcurl debug:    %s\n", $curl_debug?"ON":"OFF");
-    printf("* valgrind:         %s\n", $valgrind?"ON":"OFF");
+    printf("* Server SSL:     %s\n", $stunnel?"ON":"OFF");
+    printf("* libcurl SSL:    %s\n", $ssl_version?"ON":"OFF");
+    printf("* libcurl debug:  %s\n", $curl_debug?"ON":"OFF");
+    printf("* valgrind:       %s\n", $valgrind?"ON":"OFF");
+    printf("* HTTP IPv6       %s\n", $http_ipv6?"ON":"OFF");
+
+    printf("* HTTP port:      %d\n", $HTTPPORT);
+    printf("* FTP port:       %d\n", $FTPPORT);
+    if($stunnel) {
+        printf("* FTPS port:      %d\n", $FTPSPORT);
+        printf("* HTTPS port:     %d\n", $HTTPSPORT);
+    }
+    if($http_ipv6) {
+        printf("* HTTP IPv6 port: %d\n", $HTTP6PORT);
+    }
     print "***************************************** \n";
 }
 
 #######################################################################
-# substitute the variable stuff into either a joined up file or 
+# substitute the variable stuff into either a joined up file or
 # a command, in either case passed by reference
 #
 sub subVariables {
   my ($thing) = @_;
   $$thing =~ s/%HOSTIP/$HOSTIP/g;
-  $$thing =~ s/%HOSTPORT/$HOSTPORT/g;
-  $$thing =~ s/%HTTPPORT/$HOSTPORT/g;
+  $$thing =~ s/%HTTPPORT/$HTTPPORT/g;
+  $$thing =~ s/%HOST6IP/$HOST6IP/g;
+  $$thing =~ s/%HTTP6PORT/$HTTP6PORT/g;
   $$thing =~ s/%HTTPSPORT/$HTTPSPORT/g;
   $$thing =~ s/%FTPPORT/$FTPPORT/g;
   $$thing =~ s/%FTPSPORT/$FTPSPORT/g;
@@ -777,6 +867,15 @@ sub subVariables {
   $$thing =~ s/%PWD/$pwd/g;
 }
 
+sub fixarray {
+    my @in = @_;
+
+    for(@in) {
+        subVariables \$_;
+    }
+    return @in;
+}
+
 #######################################################################
 # Run a single specified test case
 #
@@ -786,7 +885,6 @@ sub singletest {
 
     my @what;
     my $why;
-    my $serverproblem;
 
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
@@ -794,14 +892,12 @@ sub singletest {
             # this is not a test
             print "RUN: $testnum doesn't look like a test case!\n";
         }
-        $serverproblem = 100;
+        $why = "no test";
     }
     else {
         @what = getpart("client", "features");
     }
 
-    printf("test %03d...", $testnum);
-    
     for(@what) {
         my $f = $_;
         $f =~ s/\s//g;
@@ -821,51 +917,62 @@ sub singletest {
                 next;
             }
         }
+        elsif($f eq "idn") {
+            if($has_idn) {
+                next;
+            }
+        }
+        elsif($f eq "ipv6") {
+            if($has_ipv6) {
+                next;
+            }
+        }
+        elsif($f eq "libz") {
+            if($has_libz) {
+                next;
+            }
+        }
+        elsif($f eq "getrlimit") {
+            if($has_getrlimit) {
+                next;
+            }
+        }
 
         $why = "curl lacks $f support";
-        $serverproblem = 15; # set it here
         last;
     }
 
-    if(!$serverproblem) {
-        $serverproblem = serverfortest($testnum);
+    if(!$why) {
+        $why = serverfortest($testnum);
     }
 
-    if($serverproblem) {
-        # there's a problem with the server, don't run
-        # this particular server, but count it as "skipped"
-        if($serverproblem == 2) {
-            $why = "server problems";
-        }
-        elsif($serverproblem == 100) {
-            $why = "no test";
-        }
-        elsif($serverproblem == 99) {
-            $why = "bad test";
-        }
-        elsif($serverproblem == 15) {
-            # set above, a lacking prereq
-        }
-        elsif($serverproblem == 1) {
-            $why = "no HTTPS server";
-        }
-        elsif($serverproblem == 3) {
-            $why = "no FTPS server";
-        }
-        else {
-            $why = "unfulfilled requirements";
+    if(!$why) {
+        my @precheck = getpart("client", "precheck");
+        my $cmd = $precheck[0];
+        chomp $cmd;
+        if($cmd) {
+            my @o = `$cmd 2>/dev/null`;
+            if($o[0]) {
+                $why = $o[0];
+                chomp $why;
+            }
         }
+    }
+
+    if($why) {
+        # there's a problem, count it as "skipped"
         $skipped++;
         $skipped{$why}++;
         $teststat[$testnum]=$why; # store reason for this test case
-        
-        print "SKIPPED\n";
+
         if(!$short) {
-            print "* Test $testnum: $why\n";
+            printf "test %03d SKIPPED: $why\n", $testnum;
         }
 
         return -1;
     }
+    printf("test %03d...", $testnum);
+
 
     # extract the reply data
     my @reply = getpart("reply", "data");
@@ -880,22 +987,22 @@ sub singletest {
             # of the datacheck
             chomp($replycheck[$#replycheck]);
         }
-    
+
         @reply=@replycheck;
     }
 
     # curl command to run
-    my @curlcmd= getpart("client", "command");
+    my @curlcmd= fixarray ( getpart("client", "command") );
 
     # this is the valid protocol blurb curl should generate
-    my @protocol= getpart("verify", "protocol");
+    my @protocol= fixarray ( getpart("verify", "protocol") );
 
     # redirected stdout/stderr to these files
     $STDOUT="$LOGDIR/stdout$testnum";
     $STDERR="$LOGDIR/stderr$testnum";
 
     # if this section exists, we verify that the stdout contained this:
-    my @validstdout = getpart("verify", "stdout");
+    my @validstdout = fixarray ( getpart("verify", "stdout") );
 
     # if this section exists, we verify upload
     my @upload = getpart("verify", "upload");
@@ -979,7 +1086,7 @@ sub singletest {
         subVariables \$fileContent;
 #        print "DEBUG: writing file " . $filename . "\n";
         open OUTFILE, ">$filename";
-        binmode OUTFILE; # for crapage systems, use binary       
+        binmode OUTFILE; # for crapage systems, use binary
         print OUTFILE $fileContent;
         close OUTFILE;
     }
@@ -1024,31 +1131,53 @@ sub singletest {
     }
 
     if($valgrind) {
-        $CMDLINE = "valgrind --leak-check=yes --logfile=log/valgrind$testnum -q $CMDLINE";
+        $CMDLINE = "valgrind ".$valgrind_tool."--leak-check=yes --logfile=log/valgrind$testnum $CMDLINE";
     }
 
     $CMDLINE .= "$cmdargs >>$STDOUT 2>>$STDERR";
 
     if($verbose) {
-        print "$CMDLINE\n"; 
+        print "$CMDLINE\n";
     }
 
     print CMDLOG "$CMDLINE\n";
 
+    unlink("core");
+
+    my $dumped_core;
     my $cmdres;
-    # run the command line we built
+
+    my @precommand= getpart("client", "precommand");
+    if($precommand[0]) {
+        # this is pure perl to eval!
+        my $code = join("", @precommand);
+        eval $code;
+        if($@) {
+            print "perl: $code\n";
+            print "precommand: $@";
+            exit;
+        }
+    }
+
     if($gdbthis) {
         open(GDBCMD, ">log/gdbcmd");
         print GDBCMD "set args $cmdargs\n";
         print GDBCMD "show args\n";
         close(GDBCMD);
+    }
+    # run the command line we built
+    if ($torture) {
+        return torture($CMDLINE,
+                       "gdb --directory libtest $DBGCURL -x log/gdbcmd");
+    }
+    elsif($gdbthis) {
         system("gdb --directory libtest $DBGCURL -x log/gdbcmd");
         $cmdres=0; # makes it always continue after a debugged run
     }
     else {
         $cmdres = system("$CMDLINE");
         my $signal_num  = $cmdres & 127;
-        my $dumped_core = $cmdres & 128;
+        $dumped_core = $cmdres & 128;
 
         if(!$anyway && ($signal_num || $dumped_core)) {
             $cmdres = 1000;
@@ -1057,6 +1186,24 @@ sub singletest {
             $cmdres /= 256;
         }
     }
+    if(!$dumped_core) {
+        if(-r "core") {
+            # there's core file present now!
+            $dumped_core = 1;
+        }
+    }
+
+    if($dumped_core) {
+        print "core dumped!\n";
+        if(0 && $gdb) {
+            print "running gdb for post-mortem analysis:\n";
+            open(GDBCMD, ">log/gdbcmd2");
+            print GDBCMD "bt\n";
+            close(GDBCMD);
+            system("gdb --directory libtest -x log/gdbcmd2 -batch $DBGCURL core ");
+     #       unlink("log/gdbcmd2");
+        }
+    }
 
     # remove the special FTP command file after each test!
     unlink($FTPDCMD);
@@ -1084,7 +1231,7 @@ sub singletest {
     }
 
     my %replyattr = getpartattr("reply", "data");
-    if(!$replyattr{'nocheck'} && @reply) {
+    if(!$replyattr{'nocheck'} && (@reply || $replyattr{'sendzero'})) {
         # verify the received data
         my @out = loadarray($CURLOUT);
         $res = compare("data", \@out, \@reply);
@@ -1109,8 +1256,22 @@ sub singletest {
     }
 
     if(@protocol) {
-        # verify the sent request
-        my @out = loadarray($SERVERIN);
+        my @out;
+        my $retry = 5;
+
+        # Verify the sent request. Sometimes, like in test 513 on some hosts,
+        # curl will return back faster than the server writes down the request
+        # to its file, so we might need to wait here for a while to see if the
+        # file gets written a bit later.
+
+        while($retry--) {
+            @out = loadarray($SERVERIN);
+
+            if(!$out[0]) {
+                # nothing there yet, wait a while and try again
+                sleep(1);
+            }
+        }
 
         # what to cut off from the live protocol sent by curl
         my @strip = getpart("verify", "strip");
@@ -1127,11 +1288,22 @@ sub singletest {
         }
 
         for(@strip) {
-            # strip all patterns from both arrays
+            # strip off all lines that match the patterns from both arrays
+            chomp $_;
             @out = striparray( $_, \@out);
             @protstrip= striparray( $_, \@protstrip);
         }
 
+        # what parts to cut off from the protocol
+        my @strippart = getpart("verify", "strippart");
+        my $strip;
+        for $strip (@strippart) {
+            chomp $strip;
+            for(@out) {
+                eval $strip;
+            }
+        }
+
         $res = compare("protocol", \@out, \@protstrip);
         if($res) {
             return 1;
@@ -1159,31 +1331,29 @@ sub singletest {
         }
         if(!$short) {
             print " output OK";
-        }        
+        }
     }
 
-    if($errorcode || $cmdres) {
-        if($errorcode == $cmdres) {
-            $errorcode =~ s/\n//;
-            if($verbose) {
-                print " received errorcode $errorcode OK";
-            }
-            elsif(!$short) {
-                print " error OK";
-            }
+    if($errorcode == $cmdres) {
+        $errorcode =~ s/\n//;
+        if($verbose) {
+            print " received exitcode $errorcode OK";
         }
-        else {
-            if(!$short) {
-                print "curl returned $cmdres, ".(0+$errorcode)." was expected\n";
-            }
-            print " error FAILED\n";
-            return 1;
+        elsif(!$short) {
+            print " exit OK";
         }
     }
+    else {
+        if(!$short) {
+            print "\ncurl returned $cmdres, ".(0+$errorcode)." was expected\n";
+        }
+        print " exit FAILED\n";
+        return 1;
+    }
 
     # the test succeeded, remove all log files
     if(!$keepoutfiles) {
-        cleardir($LOGDIR);        
+        cleardir($LOGDIR);
     }
 
     unlink($FTPDCMD); # remove the instructions for this test
@@ -1226,6 +1396,37 @@ sub singletest {
                 }
             }
         }
+    }
+    if($valgrind) {
+        opendir(DIR, "log") ||
+            return 0; # can't open log dir
+        my @files = readdir(DIR);
+        closedir DIR;
+        my $f;
+        my $l;
+        foreach $f (@files) {
+            if($f =~ /^valgrind$testnum/) {
+                $l = $f;
+                last;
+            }
+        }
+        my $leak;
+        open(VAL, "<$l");
+        while(<VAL>) {
+            if($_ =~ /definitely lost: (\d*) bytes/) {
+                $leak = $1;
+                last;
+            }
+        }
+        close(VAL);
+        if($leak) {
+            print " valgrind ERROR ";
+        }
+        elsif(!$short) {
+            print " valgrind OK";
+        }
+
+
     }
     if($short) {
         print "OK";
@@ -1248,17 +1449,19 @@ sub stopservers {
 #######################################################################
 # startservers() starts all the named servers
 #
+# Returns: string with error reason or blank for success
+
 sub startservers {
     my @what = @_;
     my $pid;
     for(@what) {
         my $what = lc($_);
-        $what =~ s/[^a-z]//g;
+        $what =~ s/[^a-z0-9-]//g;
         if($what eq "ftp") {
             if(!$run{'ftp'}) {
                 $pid = runftpserver($verbose);
                 if($pid <= 0) {
-                    return 2; # error starting it
+                    return "failed starting FTP server";
                 }
                 printf ("* pid ftp => %-5d\n", $pid) if($verbose);
                 $run{'ftp'}=$pid;
@@ -1268,23 +1471,37 @@ sub startservers {
             if(!$run{'http'}) {
                 $pid = runhttpserver($verbose);
                 if($pid <= 0) {
-                    return 2; # error starting
-                } 
+                    return "failed starting HTTP server";
+                }
                 printf ("* pid http => %-5d\n", $pid) if($verbose);
                 $run{'http'}=$pid;
             }
         }
+        elsif($what eq "http-ipv6") {
+            if(!$run{'http-ipv6'}) {
+                $pid = runhttpserver($verbose, "IPv6");
+                if($pid <= 0) {
+                    return "failed starting IPv6 HTTP server";
+                }
+                printf ("* pid http-ipv6 => %-5d\n", $pid) if($verbose);
+                $run{'http-ipv6'}=$pid;
+            }
+        }
         elsif($what eq "ftps") {
-            if(!$stunnel || !$ssl_version) {
+            if(!$stunnel) {
                 # we can't run ftps tests without stunnel
-                # or if libcurl is SSL-less
-                return 3;
+                return "no stunnel";
+            }
+            if(!$ssl_version) {
+                # we can't run ftps tests if libcurl is SSL-less
+                return "curl lacks SSL support";
             }
             if(!$run{'ftp'}) {
                 $pid = runftpserver($verbose);
                 if($pid <= 0) {
-                    return 2; # error starting it
+                    return "failed starting FTP server";
                 }
+                printf ("* pid ftp => %-5d\n", $pid) if($verbose);
                 $run{'ftp'}=$pid;
             }
             if(!$run{'ftps'}) {
@@ -1292,7 +1509,7 @@ sub startservers {
 
                 $pid = runftpsserver($verbose);
                 if($pid <= 0) {
-                    return 2;
+                    return "failed starting FTPS server (stunnel)";
                 }
                 printf ("* pid ftps => %-5d\n", $pid) if($verbose);
                 $run{'ftps'}=$pid;
@@ -1302,28 +1519,34 @@ sub startservers {
             # we support it but have no server!
         }
         elsif($what eq "https") {
-            if(!$stunnel || !$ssl_version) {
-                # we can't run https tests without stunnel
-                # or if libcurl is SSL-less
-                return 1;
+            if(!$stunnel) {
+                # we can't run ftps tests without stunnel
+                return "no stunnel";
             }
+            if(!$ssl_version) {
+                # we can't run ftps tests if libcurl is SSL-less
+                return "curl lacks SSL support";
+            }
+
             if(!$run{'http'}) {
                 $pid = runhttpserver($verbose);
                 if($pid <= 0) {
-                    return 2; # problems starting server
+                    return "failed starting HTTP server";
                 }
+                printf ("* pid http => %-5d\n", $pid) if($verbose);
                 $run{'http'}=$pid;
             }
             if(!$run{'https'}) {
                 $pid = runhttpsserver($verbose);
                 if($pid <= 0) {
-                    return 2;
+                    return "failed starting HTTPS server (stunnel)";
                 }
                 printf ("* pid https => %-5d\n", $pid) if($verbose);
                 $run{'https'}=$pid;
             }
         }
         elsif($what eq "none") {
+            print "* starts no server\n" if ($verbose);
         }
         else {
             warn "we don't support a server for $what";
@@ -1337,12 +1560,8 @@ sub startservers {
 # specified test case. This is a useful design when we run single tests as not
 # all servers need to run then!
 #
-# Returns:
-# 100 if this is not a test case
-# 99  if this test case has no servers specified
-# 3   if this test is skipped due to no FTPS server
-# 2   if one of the required servers couldn't be started
-# 1   if this test is skipped due to no HTTPS server
+# Returns: a string, blank if everything is fine or a reason why it failed
+#
 
 sub serverfortest {
     my ($testnum)=@_;
@@ -1353,14 +1572,14 @@ sub serverfortest {
             # this is not a test
             print "$testnum doesn't look like a test case!\n";
         }
-        return 100;
+        return "no test";
     }
 
     my @what = getpart("client", "server");
 
     if(!$what[0]) {
         warn "Test case $testnum has no server(s) specified!";
-        return 99;
+        return "no server specified";
     }
 
     return &startservers(@what);
@@ -1379,12 +1598,12 @@ do {
         $verbose=1;
     }
     elsif ($ARGV[0] eq "-c") {
-        # use this path to curl instead of default        
+        # use this path to curl instead of default
         $CURL=$ARGV[1];
         shift @ARGV;
     }
     elsif ($ARGV[0] eq "-d") {
-        # have the servers display protocol output 
+        # have the servers display protocol output
         $debugprotocol=1;
     }
     elsif ($ARGV[0] eq "-g") {
@@ -1403,12 +1622,12 @@ do {
         # torture
         $torture=1;
         my $xtra = $1;
-        if($xtra =~ s/^(\d+)//) {
-            $tortnum = $1;
-        }
+
         if($xtra =~ s/(\d+)$//) {
             $tortalloc = $1;
         }
+        # we undef valgrind to make this fly in comparison
+        undef $valgrind;
     }
     elsif($ARGV[0] eq "-a") {
         # continue anyway, even if a test fail
@@ -1478,6 +1697,13 @@ if($valgrind) {
     }
 }
 
+$HTTPPORT =  $base + 0; # HTTP server port
+$HTTPSPORT = $base + 1; # HTTPS server port
+$FTPPORT =   $base + 2; # FTP server port
+$FTPSPORT =  $base + 3; # FTPS server port
+$HTTP6PORT = $base + 4; # HTTP IPv6 server port (different IP protocol
+                        # but we follow the same port scheme anyway)
+
 #######################################################################
 # Output curl version and host info being tested
 #
@@ -1504,7 +1730,7 @@ if ( $TESTCASES eq "all") {
 
     $TESTCASES=""; # start with no test cases
 
-    # cut off everything but the digits 
+    # cut off everything but the digits
     for(@cmds) {
         $_ =~ s/[a-z\/\.]*//g;
     }
@@ -1521,11 +1747,6 @@ open(CMDLOG, ">$CURLLOG") ||
     print "can't log command lines to $CURLLOG\n";
 
 #######################################################################
-# Torture the memory allocation system and checks
-#
-if($torture) {
-    &torture();
-}
 
 sub displaylogcontent {
     my ($file)=@_;
@@ -1548,6 +1769,9 @@ sub displaylogs {
     foreach $log (sort @logs) {
         # the log file is not "." or ".." and contains more than zero bytes
         if(($log !~ /\.(\.|)$/) && -s "$LOGDIR/$log") {
+            if($log =~ /^\.nfs/) {
+                next;
+            }
             print "== Start of file $log\n";
             displaylogcontent("$LOGDIR/$log");
             print "== End of file $log\n";
